@@ -18,15 +18,15 @@ case "$repo" in
 		echo -e 'repos:\n  - knot-dns-latest' > repos.yaml
 		;;
 	*)
-		echo "Unknown repo, choose devel|latest"
+		echo "Unknown repo, choose devel|latest|testing"
 		exit 1
 		;;
 esac
 
 cd "$distro"
-vagrant destroy &>/dev/null
+vagrant destroy -f &>/dev/null
 vagrant up
 ret=$?
-vagrant destroy &>/dev/null
+vagrant destroy -f &>/dev/null
 exit $ret
 
