@@ -1,4 +1,4 @@
-/*  Copyright (C) 2011 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -154,6 +154,7 @@ int32_t base64_encode_alloc(const uint8_t  *in,
 	int32_t ret = base64_encode(in, in_len, *out, out_len);
 	if (ret < 0) {
 		free(*out);
+		*out = NULL;
 	}
 
 	return ret;
@@ -215,8 +216,10 @@ int32_t base64_decode(const uint8_t  *in,
 		switch (pad_len) {
 		case 0:
 			bin[2] = (c3 << 6) + c4;
+			// FALLTHROUGH
 		case 1:
 			bin[1] = (c2 << 4) + (c3 >> 2);
+			// FALLTHROUGH
 		case 2:
 			bin[0] = (c1 << 2) + (c2 >> 4);
 		}
@@ -262,6 +265,7 @@ int32_t base64_decode_alloc(const uint8_t  *in,
 	int32_t ret = base64_decode(in, in_len, *out, out_len);
 	if (ret < 0) {
 		free(*out);
+		*out = NULL;
 	}
 
 	return ret;
