@@ -28,11 +28,3 @@
 int conf_migrate(
 	conf_t *conf
 );
-
-#define C_MOD_ONLINE_SIGN "\x0f""mod-online-sign"
-extern const yp_item_t schema_mod_online_sign[];
-int check_mod_online_sign(knotd_conf_check_args_t *args);
-
-#define C_MOD_SYNTH_RECORD "\x10""mod-synth-record"
-extern const yp_item_t schema_mod_synth_record[];
-int check_mod_synth_record(knotd_conf_check_args_t *args);
