@@ -1,4 +1,4 @@
-/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2019 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -237,8 +237,7 @@ static int zone_status(zone_t *zone, ctl_args_t *args)
 	if (MATCH_OR_FILTER(args, CTL_FILTER_STATUS_EVENTS)) {
 		for (zone_event_type_t i = 0; i < ZONE_EVENT_COUNT; i++) {
 			// Events not worth showing or used elsewhere.
-			if (i == ZONE_EVENT_LOAD || i == ZONE_EVENT_UFREEZE ||
-			    i == ZONE_EVENT_UTHAW) {
+			if (i == ZONE_EVENT_UFREEZE || i == ZONE_EVENT_UTHAW) {
 				continue;
 			}
 
@@ -1383,9 +1382,12 @@ static int server_status(ctl_args_t *args)
 	if (strcasecmp(type, "version") == 0) {
 		ret = snprintf(buff, sizeof(buff), "Version: %s", PACKAGE_VERSION);
 	} else if (strcasecmp(type, "workers") == 0) {
+		int running_bkg_wrk, wrk_queue;
+		worker_pool_status(args->server->workers, &running_bkg_wrk, &wrk_queue);
 		ret = snprintf(buff, sizeof(buff), "UDP workers: %zu, TCP workers %zu, "
-		               "background workers: %zu", conf_udp_threads(conf()),
-		               conf_tcp_threads(conf()), conf_bg_threads(conf()));
+		               "background workers: %zu (running: %d, pending: %d)",
+		               conf_udp_threads(conf()), conf_tcp_threads(conf()),
+		               conf_bg_threads(conf()), running_bkg_wrk, wrk_queue);
 	} else if (strcasecmp(type, "configure") == 0) {
 		ret = snprintf(buff, sizeof(buff), "%s", CONFIGURE_SUMMARY);
 	} else {
