@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -170,7 +170,7 @@ void kasp_db_close(kasp_db_t **db)
 
 static knot_db_val_t make_key(keyclass_t kclass, const knot_dname_t *dname, const char *str)
 {
-	size_t dnlen = (dname == NULL ? 0 : knot_dname_size((const knot_dname_t *)dname));
+	size_t dnlen = knot_dname_size(dname);
 	size_t slen = (str == NULL ? 0 : strlen(str) + 1);
 	knot_db_val_t res = { .len = 1 + dnlen + slen, .data = malloc(1 + dnlen + slen) };
 	if (res.data != NULL) {
@@ -195,7 +195,7 @@ static char *keyid_fromkey(const knot_db_val_t *key)
 	if (key->len < 2 || *(uint8_t *)key->data != KASPDBKEY_PARAMS) {
 		return NULL;
 	}
-	size_t skip = knot_dname_size((const knot_dname_t *)key->data + 1);
+	size_t skip = knot_dname_size((const uint8_t *)key->data + 1);
 	return (key->len < skip + 2 ? NULL : strdup(key->data + skip + 1));
 }
 
@@ -204,7 +204,7 @@ static bool check_key_zone(const knot_db_val_t *key, const knot_dname_t *zone_na
 	if (key->len < 2 || *(uint8_t *)key->data == KASPDBKEY_POLICYLAST) {
 		return false;
 	}
-	return (knot_dname_cmp(key->data + 1, zone_name) == 0);
+	return knot_dname_is_equal(key->data + 1, zone_name);
 }
 
 static int serialize_key_params(const key_params_t *params, const knot_dname_t *dname, knot_db_val_t *key, knot_db_val_t *val)
@@ -471,26 +471,18 @@ int kasp_db_delete_all(kasp_db_t *db, const knot_dname_t *zone_name)
 	WALK_LIST(n, allkeys) {
 		key_params_t *parm = n->d;
 		knot_db_val_t key = make_key(KASPDBKEY_PARAMS, zone_name, parm->id);
-		ret = db_api->del(txn, &key);
+		(void)db_api->del(txn, &key);
 		free_key(&key);
 		free(parm->id);
 		free(parm->public_key.data);
 		memset(parm, 0, sizeof(*parm));
-		if (ret != KNOT_EOK) {
-			break;
-		}
 	}
 	ptrlist_deep_free(&allkeys, NULL);
 
-	if (ret == KNOT_EOK) {
-		knot_db_val_t key = make_key(KASPDBKEY_NSEC3SALT, zone_name, NULL);
-		ret = db_api->del(txn, &key);
+	for (keyclass_t keyclass = KASPDBKEY_NSEC3SALT; keyclass <= KASPDBKEY_LASTSIGNEDSERIAL; keyclass++) {
+		knot_db_val_t key = make_key(keyclass, zone_name, NULL);
+		(void)db_api->del(txn, &key);
 		free_key(&key);
-		if (ret == KNOT_EOK) {
-			key = make_key(KASPDBKEY_NSEC3TIME, zone_name, NULL);
-			ret = db_api->del(txn, &key);
-			free_key(&key);
-		}
 	}
 
 	with_txn_end(NULL, NULL);
@@ -657,6 +649,15 @@ int kasp_db_get_policy_last(kasp_db_t *db, const char *policy_string, knot_dname
 				free(*lp_zone);
 				free(*lp_keyid);
 				ret = KNOT_ENOMEM;
+			} else {
+				// check that the shared key ID really exists
+				key = make_key(KASPDBKEY_PARAMS, *lp_zone, *lp_keyid);
+				ret = db_api->find(txn, &key, &val, 0);
+				free_key(&key);
+				if (ret != KNOT_EOK) {
+					free(*lp_zone);
+					free(*lp_keyid);
+				}
 			}
 		}
 	}
@@ -722,7 +723,7 @@ int kasp_db_list_zones(kasp_db_t *db, list_t *dst)
 			ptrnode_t *n;
 			WALK_LIST(n, *dst) {
 				knot_dname_t *exist_dn = (knot_dname_t *)n->d;
-				if (knot_dname_cmp(key_dn, exist_dn) == 0) {
+				if (knot_dname_is_equal(key_dn, exist_dn)) {
 					key_dn = NULL;
 					break;
 				}
