@@ -40,11 +40,12 @@ int knot_dnssec_key_rollover(kdnssec_ctx_t *ctx, zone_sign_reschedule_t *resched
 /*!
  * \brief Set the submitted KSK to active state and the active one to retired
  *
- * \param ctx zone signing context
+ * \param ctx           Zone signing context.
+ * \param retire_delay  Retire event delay.
  *
  * \return KNOT_E*
  */
-int knot_dnssec_ksk_sbm_confirm(kdnssec_ctx_t *ctx);
+int knot_dnssec_ksk_sbm_confirm(kdnssec_ctx_t *ctx, uint32_t retire_delay);
 
 /*!
  * \brief Is there a key in sumbmission phase?
