@@ -1,4 +1,4 @@
-/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2019 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,7 +25,6 @@
 #include "knot/dnssec/rrset-sign.h"
 #include "knot/dnssec/zone-sign.h"
 #include "libknot/libknot.h"
-#include "contrib/dynarray.h"
 #include "contrib/macros.h"
 #include "contrib/wire_ctx.h"
 
@@ -132,31 +131,29 @@ static bool valid_signature_exists(const knot_rrset_t *covered,
 }
 
 /*!
- * \brief Check if valid signature exist for all keys for a given RR set.
+ * \brief Check if valid signature exists for all keys for a given RR set.
  *
  * \param covered    RR set with covered records.
  * \param rrsigs     RR set with RRSIGs.
- * \param zone_keys  Zone keys.
- * \param policy     DNSSEC policy.
+ * \param sign_ctx   Local zone signing context.
  *
  * \return Valid signature exists for every key.
  */
 static bool all_signatures_exist(const knot_rrset_t *covered,
                                  const knot_rrset_t *rrsigs,
-                                 const zone_keyset_t *zone_keys,
-                                 const kdnssec_ctx_t *dnssec_ctx)
+                                 zone_sign_ctx_t *sign_ctx)
 {
-	assert(!knot_rrset_empty(covered));
-	assert(zone_keys);
+	assert(sign_ctx);
 
-	for (int i = 0; i < zone_keys->count; i++) {
-		zone_key_t *key = &zone_keys->keys[i];
+	for (int i = 0; i < sign_ctx->count; i++) {
+		zone_key_t *key = &sign_ctx->keys[i];
 		if (!knot_zone_sign_use_key(key, covered)) {
 			continue;
 		}
 
 		if (!valid_signature_exists(covered, rrsigs, key->key,
-		                            key->ctx, dnssec_ctx)) {
+		                            sign_ctx->sign_ctxs[i],
+		                            sign_ctx->dnssec_ctx)) {
 			return false;
 		}
 	}
@@ -185,8 +182,7 @@ static void note_earliest_expiration(const knot_rdata_t *rrsig, knot_time_t *exp
  *
  * \param covered     RR set with covered records.
  * \param rrsigs      RR set with RRSIGs.
- * \param zone_keys   Zone keys.
- * \param policy      DNSSEC policy.
+ * \param sign_ctx    Local zone signing context.
  * \param changeset   Changeset to be updated.
  * \param expires_at  Earliest RRSIG expiration.
  *
@@ -194,8 +190,7 @@ static void note_earliest_expiration(const knot_rdata_t *rrsig, knot_time_t *exp
  */
 static int remove_expired_rrsigs(const knot_rrset_t *covered,
                                  const knot_rrset_t *rrsigs,
-                                 const zone_keyset_t *zone_keys,
-                                 const kdnssec_ctx_t *dnssec_ctx,
+                                 zone_sign_ctx_t *sign_ctx,
                                  changeset_t *changeset,
                                  knot_time_t *expires_at)
 {
@@ -224,16 +219,17 @@ static int remove_expired_rrsigs(const knot_rrset_t *covered,
 	for (uint16_t i = 0; i < rrsig_rdata_count; i++) {
 		knot_rdata_t *rr = knot_rdataset_at(&synth_rrsig.rrs, i);
 		uint16_t keytag = knot_rrsig_key_tag(rr);
-
-		struct keyptr_dynarray keys = get_zone_keys(zone_keys, keytag);
 		int endloop = 0; // 1 - continue; 2 - break
 
-		dynarray_foreach(keyptr, zone_key_t *, key, keys) {
-			if (!(*key)->is_active) {
+		for (size_t j = 0; j < sign_ctx->count; j++) {
+			zone_key_t *key = &sign_ctx->keys[j];
+
+			if (!key->is_active || dnssec_key_get_keytag(key->key) != keytag) {
 				continue;
 			}
-			result = knot_check_signature(covered, &synth_rrsig, i,
-			                              (*key)->key, (*key)->ctx, dnssec_ctx);
+
+			result = knot_check_signature(covered, &synth_rrsig, i, key->key,
+			                              sign_ctx->sign_ctxs[j], sign_ctx->dnssec_ctx);
 			if (result == KNOT_EOK) {
 				// valid signature
 				note_earliest_expiration(rr, expires_at);
@@ -244,7 +240,6 @@ static int remove_expired_rrsigs(const knot_rrset_t *covered,
 				break;
 			}
 		}
-		keyptr_dynarray_free(&keys);
 
 		if (endloop == 2) {
 			break;
@@ -277,8 +272,7 @@ static int remove_expired_rrsigs(const knot_rrset_t *covered,
  *
  * \param covered     RR set with covered records.
  * \param rrsigs      RR set with RRSIGs.
- * \param zone_keys   Zone keys.
- * \param dnssec_ctx  DNSSEC signing context
+ * \param sign_ctx    Local zone signing context.
  * \param changeset   Changeset to be updated.
  * \param expires_at  Earliest RRSIG expiration.
  *
@@ -286,26 +280,26 @@ static int remove_expired_rrsigs(const knot_rrset_t *covered,
  */
 static int add_missing_rrsigs(const knot_rrset_t *covered,
                               const knot_rrset_t *rrsigs,
-                              const zone_keyset_t *zone_keys,
-                              const kdnssec_ctx_t *dnssec_ctx,
+                              zone_sign_ctx_t *sign_ctx,
                               changeset_t *changeset,
                               knot_time_t *expires_at)
 {
 	assert(!knot_rrset_empty(covered));
-	assert(zone_keys);
+	assert(sign_ctx);
 	assert(changeset);
 
 	int result = KNOT_EOK;
 	knot_rrset_t to_add;
 	knot_rrset_init_empty(&to_add);
 
-	for (int i = 0; i < zone_keys->count; i++) {
-		const zone_key_t *key = &zone_keys->keys[i];
+	for (size_t i = 0; i < sign_ctx->count; i++) {
+		const zone_key_t *key = &sign_ctx->keys[i];
 		if (!knot_zone_sign_use_key(key, covered)) {
 			continue;
 		}
 
-		if (valid_signature_exists(covered, rrsigs, key->key, key->ctx, dnssec_ctx)) {
+		if (valid_signature_exists(covered, rrsigs, key->key, sign_ctx->sign_ctxs[i],
+		                           sign_ctx->dnssec_ctx)) {
 			continue;
 		}
 
@@ -313,8 +307,8 @@ static int add_missing_rrsigs(const knot_rrset_t *covered,
 			to_add = create_empty_rrsigs_for(covered);
 		}
 
-		result = knot_sign_rrset(&to_add, covered, key->key, key->ctx,
-		                         dnssec_ctx, NULL, expires_at);
+		result = knot_sign_rrset(&to_add, covered, key->key, sign_ctx->sign_ctxs[i],
+		                         sign_ctx->dnssec_ctx, NULL, expires_at);
 		if (result != KNOT_EOK) {
 			break;
 		}
@@ -364,16 +358,15 @@ static int remove_rrset_rrsigs(const knot_dname_t *owner, uint16_t type,
  * \brief Drop all existing and create new RRSIGs for covered records.
  *
  * \param covered    RR set with covered records.
- * \param zone_keys  Zone keys.
- * \param policy     DNSSEC policy.
+ * \param rrsigs     Existing RRSIGs for covered RR set.
+ * \param sign_ctx   Local zone signing context.
  * \param changeset  Changeset to be updated.
  *
  * \return Error code, KNOT_EOK if successful.
  */
 static int force_resign_rrset(const knot_rrset_t *covered,
                               const knot_rrset_t *rrsigs,
-                              const zone_keyset_t *zone_keys,
-                              const kdnssec_ctx_t *dnssec_ctx,
+                              zone_sign_ctx_t *sign_ctx,
                               changeset_t *changeset)
 {
 	assert(!knot_rrset_empty(covered));
@@ -386,15 +379,15 @@ static int force_resign_rrset(const knot_rrset_t *covered,
 		}
 	}
 
-	return add_missing_rrsigs(covered, NULL, zone_keys, dnssec_ctx, changeset, NULL);
+	return add_missing_rrsigs(covered, NULL, sign_ctx, changeset, NULL);
 }
 
 /*!
  * \brief Drop all expired and create new RRSIGs for covered records.
  *
  * \param covered     RR set with covered records.
- * \param zone_keys   Zone keys.
- * \param policy      DNSSEC policy.
+ * \param rrsigs      Existing RRSIGs for covered RR set.
+ * \param sign_ctx    Local zone signing context.
  * \param changeset   Changeset to be updated.
  * \param expires_at  Current earliest expiration, will be updated.
  *
@@ -402,22 +395,20 @@ static int force_resign_rrset(const knot_rrset_t *covered,
  */
 static int resign_rrset(const knot_rrset_t *covered,
                         const knot_rrset_t *rrsigs,
-                        const zone_keyset_t *zone_keys,
-                        const kdnssec_ctx_t *dnssec_ctx,
+                        zone_sign_ctx_t *sign_ctx,
                         changeset_t *changeset,
                         knot_time_t *expires_at)
 {
 	assert(!knot_rrset_empty(covered));
 
 	// TODO this function creates some signatures twice (for checking)
-	int result = remove_expired_rrsigs(covered, rrsigs, zone_keys,
-	                                   dnssec_ctx, changeset, expires_at);
+	int result = remove_expired_rrsigs(covered, rrsigs, sign_ctx,
+	                                   changeset, expires_at);
 	if (result != KNOT_EOK) {
 		return result;
 	}
 
-	return add_missing_rrsigs(covered, rrsigs, zone_keys, dnssec_ctx,
-	                          changeset, expires_at);
+	return add_missing_rrsigs(covered, rrsigs, sign_ctx, changeset, expires_at);
 }
 
 static int remove_standalone_rrsigs(const zone_node_t *node,
@@ -456,21 +447,19 @@ static int remove_standalone_rrsigs(const zone_node_t *node,
  * \brief Update RRSIGs in a given node by updating changeset.
  *
  * \param node        Node to be signed.
- * \param zone_keys   Zone keys.
- * \param policy      DNSSEC policy.
+ * \param sign_ctx    Local zone signing context.
  * \param changeset   Changeset to be updated.
  * \param expires_at  Current earliest expiration, will be updated.
  *
  * \return Error code, KNOT_EOK if successful.
  */
 static int sign_node_rrsets(const zone_node_t *node,
-                            const zone_keyset_t *zone_keys,
-                            const kdnssec_ctx_t *dnssec_ctx,
+                            zone_sign_ctx_t *sign_ctx,
                             changeset_t *changeset,
                             knot_time_t *expires_at)
 {
 	assert(node);
-	assert(dnssec_ctx);
+	assert(sign_ctx);
 
 	int result = KNOT_EOK;
 	knot_rrset_t rrsigs = node_rrset(node, KNOT_RRTYPE_RRSIG);
@@ -485,12 +474,12 @@ static int sign_node_rrsets(const zone_node_t *node,
 			continue;
 		}
 
-		if (dnssec_ctx->rrsig_drop_existing) {
-			result = force_resign_rrset(&rrset, &rrsigs, zone_keys,
-			                            dnssec_ctx, changeset);
+		if (sign_ctx->dnssec_ctx->rrsig_drop_existing) {
+			result = force_resign_rrset(&rrset, &rrsigs,
+			                            sign_ctx, changeset);
 		} else {
-			result = resign_rrset(&rrset, &rrsigs, zone_keys,
-			                      dnssec_ctx, changeset, expires_at);
+			result = resign_rrset(&rrset, &rrsigs, sign_ctx,
+			                      changeset, expires_at);
 		}
 
 		if (result != KNOT_EOK) {
@@ -505,8 +494,7 @@ static int sign_node_rrsets(const zone_node_t *node,
  * \brief Struct to carry data for 'sign_data' callback function.
  */
 typedef struct node_sign_args {
-	const zone_keyset_t *zone_keys;
-	const kdnssec_ctx_t *dnssec_ctx;
+	zone_sign_ctx_t *sign_ctx;
 	changeset_t *changeset;
 	knot_time_t expires_at;
 } node_sign_args_t;
@@ -532,9 +520,8 @@ static int sign_node(zone_node_t **node, void *data)
 		return KNOT_EOK;
 	}
 
-	int result = sign_node_rrsets(*node, args->zone_keys, args->dnssec_ctx,
-	                              args->changeset, &args->expires_at);
-	(*node)->flags &= ~NODE_FLAGS_REMOVED_NSEC;
+	int result = sign_node_rrsets(*node, args->sign_ctx, args->changeset,
+	                              &args->expires_at);
 
 	return result;
 }
@@ -561,15 +548,19 @@ static int zone_tree_sign(zone_tree_t *tree,
 	assert(changeset);
 
 	node_sign_args_t args = {
-		.zone_keys = zone_keys,
-		.dnssec_ctx = dnssec_ctx,
+		.sign_ctx = zone_sign_ctx(zone_keys, dnssec_ctx),
 		.changeset = changeset,
 		.expires_at = knot_time_add(dnssec_ctx->now, dnssec_ctx->policy->rrsig_lifetime),
 	};
 
+	if (args.sign_ctx == NULL) {
+		return KNOT_ENOMEM;
+	}
+
 	int result = zone_tree_apply(tree, sign_node, &args);
 	*expires_at = args.expires_at;
 
+	zone_sign_ctx_free(args.sign_ctx);
 	return result;
 }
 
@@ -580,8 +571,7 @@ static int zone_tree_sign(zone_tree_t *tree,
  */
 typedef struct {
 	const zone_contents_t *zone;
-	const zone_keyset_t *zone_keys;
-	const kdnssec_ctx_t *dnssec_ctx;
+	zone_sign_ctx_t *sign_ctx;
 	changeset_t *changeset;
 	trie_t *signed_tree;
 } changeset_signing_data_t;
@@ -761,8 +751,8 @@ static int sign_changeset_wrap(knot_rrset_t *chg_rrset,
 		}
 
 		if (should_sign) {
-			return resign_rrset(&zone_rrset, &rrsigs, args->zone_keys,
-			                    args->dnssec_ctx, args->changeset, expire_at);
+			return resign_rrset(&zone_rrset, &rrsigs, args->sign_ctx,
+			                    args->changeset, expire_at);
 		} else {
 			/*
 			 * If RRSet in zone DOES have RRSIGs although we
@@ -973,7 +963,7 @@ cleanup:
 bool knot_zone_sign_use_key(const zone_key_t *key, const knot_rrset_t *covered)
 {
 	if (key == NULL || covered == NULL) {
-		return KNOT_EINVAL;
+		return false;
 	}
 
 	if (!key->is_active) {
@@ -1005,14 +995,20 @@ bool knot_zone_sign_soa_expired(const zone_contents_t *zone,
                                 const zone_keyset_t *zone_keys,
                                 const kdnssec_ctx_t *dnssec_ctx)
 {
-	assert(zone);
-	assert(zone_keys);
-	assert(dnssec_ctx);
+	if (zone == NULL || zone_keys == NULL || dnssec_ctx == NULL) {
+		return false;
+	}
 
 	knot_rrset_t soa = node_rrset(zone->apex, KNOT_RRTYPE_SOA);
-	knot_rrset_t rrsigs = node_rrset(zone->apex, KNOT_RRTYPE_RRSIG);
 	assert(!knot_rrset_empty(&soa));
-	return !all_signatures_exist(&soa, &rrsigs, zone_keys, dnssec_ctx);
+	knot_rrset_t rrsigs = node_rrset(zone->apex, KNOT_RRTYPE_RRSIG);
+	zone_sign_ctx_t *sign_ctx = zone_sign_ctx(zone_keys, dnssec_ctx);
+	if (sign_ctx == NULL) {
+		return false;
+	}
+	bool exist = all_signatures_exist(&soa, &rrsigs, sign_ctx);
+	zone_sign_ctx_free(sign_ctx);
+	return !exist;
 }
 
 static int sign_changeset(const zone_contents_t *zone,
@@ -1026,50 +1022,54 @@ static int sign_changeset(const zone_contents_t *zone,
 		return KNOT_EINVAL;
 	}
 
+	int ret = KNOT_EOK;
+
 	// Create args for wrapper function - trie for duplicate sigs
 	changeset_signing_data_t args = {
 		.zone = zone,
-		.zone_keys = zone_keys,
-		.dnssec_ctx = dnssec_ctx,
+		.sign_ctx = zone_sign_ctx(zone_keys, dnssec_ctx),
 		.changeset = out_ch,
 		.signed_tree = trie_create(NULL)
 	};
 
-	if (args.signed_tree == NULL) {
-		return KNOT_ENOMEM;
-
+	if (args.sign_ctx == NULL || args.signed_tree == NULL) {
+		ret = KNOT_ENOMEM;
+		goto cleanup;
 	}
+
 	changeset_iter_t itt;
 	changeset_iter_all(&itt, in_ch);
 
 	knot_rrset_t rr = changeset_iter_next(&itt);
 	while (!knot_rrset_empty(&rr)) {
-		int ret = sign_changeset_wrap(&rr, &args, expire_at);
+		ret = sign_changeset_wrap(&rr, &args, expire_at);
 		if (ret != KNOT_EOK) {
 			changeset_iter_clear(&itt);
-			return ret;
+			goto cleanup;
 		}
 		rr = changeset_iter_next(&itt);
 	}
 	changeset_iter_clear(&itt);
 
 	if (!knot_rrset_empty(in_ch->soa_from)) {
-		int ret = sign_changeset_wrap(in_ch->soa_from, &args, expire_at);
+		ret = sign_changeset_wrap(in_ch->soa_from, &args, expire_at);
 		if (ret != KNOT_EOK) {
-			return ret;
+			goto cleanup;
 		}
 	}
 	if (!knot_rrset_empty(in_ch->soa_to)) {
-		int ret = sign_changeset_wrap(in_ch->soa_to, &args, expire_at);
+		ret = sign_changeset_wrap(in_ch->soa_to, &args, expire_at);
 		if (ret != KNOT_EOK) {
-			return ret;
+			goto cleanup;
 		}
 	}
 
+cleanup:
 	knot_zone_clear_sorted_changes(args.signed_tree);
 	trie_free(args.signed_tree);
+	zone_sign_ctx_free(args.sign_ctx);
 
-	return KNOT_EOK;
+	return ret;
 }
 
 int knot_zone_sign_nsecs_in_changeset(const zone_keyset_t *zone_keys,
@@ -1080,6 +1080,11 @@ int knot_zone_sign_nsecs_in_changeset(const zone_keyset_t *zone_keys,
 		return KNOT_EINVAL;
 	}
 
+	zone_sign_ctx_t *sign_ctx = zone_sign_ctx(zone_keys, dnssec_ctx);
+	if (sign_ctx == NULL) {
+		return KNOT_ENOMEM;
+	}
+
 	changeset_iter_t itt;
 	changeset_iter_add(&itt, changeset);
 
@@ -1088,8 +1093,8 @@ int knot_zone_sign_nsecs_in_changeset(const zone_keyset_t *zone_keys,
 		if (rr.type == KNOT_RRTYPE_NSEC ||
 		    rr.type == KNOT_RRTYPE_NSEC3 ||
 		    rr.type == KNOT_RRTYPE_NSEC3PARAM) {
-			int ret =  add_missing_rrsigs(&rr, NULL, zone_keys,
-			                              dnssec_ctx, changeset, NULL);
+			int ret =  add_missing_rrsigs(&rr, NULL, sign_ctx,
+			                              changeset, NULL);
 			if (ret != KNOT_EOK) {
 				changeset_iter_clear(&itt);
 				return ret;
@@ -1097,7 +1102,9 @@ int knot_zone_sign_nsecs_in_changeset(const zone_keyset_t *zone_keys,
 		}
 		rr = changeset_iter_next(&itt);
 	}
+
 	changeset_iter_clear(&itt);
+	zone_sign_ctx_free(sign_ctx);
 
 	return KNOT_EOK;
 }
@@ -1122,12 +1129,6 @@ bool knot_zone_sign_rr_should_be_signed(const zone_node_t *node,
 		}
 	}
 
-	// These RRs have their signatures stored in changeset already
-	if ((node->flags & NODE_FLAGS_REMOVED_NSEC) &&
-	    (rrset->type == KNOT_RRTYPE_NSEC || rrset->type == KNOT_RRTYPE_NSEC3)) {
-		return false;
-	}
-
 	return true;
 }
 
@@ -1150,8 +1151,7 @@ int knot_zone_sign_update(zone_update_t *update,
 
 	/* Check if the UPDATE changed DNSKEYs or NSEC3PARAM.
 	 * If so, we have to sign the whole zone. */
-	const bool full_sign = changeset_empty(&update->change) ||
-			       apex_dnssec_changed(update);
+	const bool full_sign = apex_dnssec_changed(update);
 	if (full_sign) {
 		ret = knot_zone_sign(update, zone_keys, dnssec_ctx, expire_at);
 	} else {
@@ -1180,10 +1180,16 @@ int knot_zone_sign_soa(zone_update_t *update,
 	changeset_t ch;
 	int ret = changeset_init(&ch, update->zone->name);
 	if (ret == KNOT_EOK) {
-		ret = force_resign_rrset(&soa_to, &soa_rrsig, zone_keys, dnssec_ctx, &ch);
+		zone_sign_ctx_t *sign_ctx = zone_sign_ctx(zone_keys, dnssec_ctx);
+		if (sign_ctx == NULL) {
+			changeset_clear(&ch);
+			return KNOT_ENOMEM;
+		}
+		ret = force_resign_rrset(&soa_to, &soa_rrsig, sign_ctx, &ch);
 		if (ret == KNOT_EOK) {
 			ret = zone_update_apply_changeset_fix(update, &ch);
 		}
+		zone_sign_ctx_free(sign_ctx);
 	}
 	changeset_clear(&ch);
 	return ret;
