@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -286,7 +286,7 @@ static int reverse_rr(char *addr_str, const synth_template_t *tpl, knot_pkt_t *p
 	}
 
 	rr->type = KNOT_RRTYPE_PTR;
-	knot_rrset_add_rdata(rr, ptrname, knot_dname_size(ptrname), tpl->ttl, &pkt->mm);
+	knot_rrset_add_rdata(rr, ptrname, knot_dname_size(ptrname), &pkt->mm);
 
 	return KNOT_EOK;
 }
@@ -302,12 +302,12 @@ static int forward_rr(char *addr_str, const synth_template_t *tpl, knot_pkt_t *p
 		rr->type = KNOT_RRTYPE_AAAA;
 		const struct sockaddr_in6* ip = (const struct sockaddr_in6*)&query_addr;
 		knot_rrset_add_rdata(rr, (const uint8_t *)&ip->sin6_addr,
-		                     sizeof(struct in6_addr), tpl->ttl, &pkt->mm);
+		                     sizeof(struct in6_addr), &pkt->mm);
 	} else if (addr_family == AF_INET) {
 		rr->type = KNOT_RRTYPE_A;
 		const struct sockaddr_in* ip = (const struct sockaddr_in*)&query_addr;
 		knot_rrset_add_rdata(rr, (const uint8_t *)&ip->sin_addr,
-		                     sizeof(struct in_addr), tpl->ttl, &pkt->mm);
+		                     sizeof(struct in_addr), &pkt->mm);
 	} else {
 		return KNOT_EINVAL;
 	}
@@ -318,7 +318,8 @@ static int forward_rr(char *addr_str, const synth_template_t *tpl, knot_pkt_t *p
 static knot_rrset_t *synth_rr(char *addr_str, const synth_template_t *tpl, knot_pkt_t *pkt,
                               knotd_qdata_t *qdata, int addr_family)
 {
-	knot_rrset_t *rr = knot_rrset_new(qdata->name, 0, KNOT_CLASS_IN, &pkt->mm);
+	knot_rrset_t *rr = knot_rrset_new(qdata->name, 0, KNOT_CLASS_IN, tpl->ttl,
+	                                  &pkt->mm);
 	if (rr == NULL) {
 		return NULL;
 	}
@@ -332,7 +333,7 @@ static knot_rrset_t *synth_rr(char *addr_str, const synth_template_t *tpl, knot_
 	}
 
 	if (ret != KNOT_EOK) {
-		knot_rrset_free(&rr, &pkt->mm);
+		knot_rrset_free(rr, &pkt->mm);
 		return NULL;
 	}
 
