@@ -14,6 +14,8 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include <netinet/in.h>
+
 #include "knot/include/module.h"
 
 static knotd_in_state_t whoami_query(knotd_in_state_t state, knot_pkt_t *pkt,
@@ -57,7 +59,7 @@ static knotd_in_state_t whoami_query(knotd_in_state_t state, knot_pkt_t *pkt,
 	 * length into 'rdata' and 'len_rdata'.
 	 */
 	const void *rdata = NULL;
-	size_t len_rdata = 0;
+	uint16_t len_rdata = 0;
 	if (query_source->ss_family == AF_INET && qtype == KNOT_RRTYPE_A) {
 		const struct sockaddr_in *sai = (struct sockaddr_in *)query_source;
 		rdata = &sai->sin_addr.s_addr;
@@ -71,36 +73,28 @@ static knotd_in_state_t whoami_query(knotd_in_state_t state, knot_pkt_t *pkt,
 		return state;
 	}
 
-	/* Sanity check, since knot_rrset_add_rdata() takes a uint16_t length
-	 * parameter.
-	 */
-	if (len_rdata > UINT16_MAX) {
-		return state;
-	}
-
 	/* Synthesize the response RRset. */
 
+	/* TTL is taken from the TTL of the SOA record. */
+	knot_rrset_t soa = knotd_qdata_zone_apex_rrset(qdata, KNOT_RRTYPE_SOA);
+
 	/* Owner name, type, and class are taken from the question. */
-	knot_rrset_t *rrset = knot_rrset_new(qname, qtype, qclass, &pkt->mm);
+	knot_rrset_t *rrset = knot_rrset_new(qname, qtype, qclass, soa.ttl, &pkt->mm);
 	if (rrset == NULL) {
 		return KNOTD_IN_STATE_ERROR;
 	}
 
-	/* TTL is taken from the TTL of the SOA record. */
-	knot_rrset_t soa = knotd_qdata_zone_apex_rrset(qdata, KNOT_RRTYPE_SOA);
-	uint32_t ttl = knot_rrset_ttl(&soa);
-
 	/* Record data is the query source address. */
-	int ret = knot_rrset_add_rdata(rrset, rdata, len_rdata, ttl, &pkt->mm);
+	int ret = knot_rrset_add_rdata(rrset, rdata, len_rdata, &pkt->mm);
 	if (ret != KNOT_EOK) {
-		knot_rrset_free(&rrset, &pkt->mm);
+		knot_rrset_free(rrset, &pkt->mm);
 		return KNOTD_IN_STATE_ERROR;
 	}
 
 	/* Add the new RRset to the response packet. */
 	ret = knot_pkt_put(pkt, KNOT_COMPR_HINT_QNAME, rrset, KNOT_PF_FREE);
 	if (ret != KNOT_EOK) {
-		knot_rrset_free(&rrset, &pkt->mm);
+		knot_rrset_free(rrset, &pkt->mm);
 		return KNOTD_IN_STATE_ERROR;
 	}
 
