@@ -1,4 +1,4 @@
-/*  Copyright (C) 2015 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,4 +24,4 @@
  *
  * \return KNOT_STATE_* processing states
  */
-int axfr_process_query(knot_pkt_t *pkt, struct query_data *qdata);
+int axfr_process_query(knot_pkt_t *pkt, knotd_qdata_t *qdata);
