@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -97,15 +97,15 @@ static int create_txt_rrset(knot_rrset_t *rrset, const knot_dname_t *owner,
 		return KNOT_ENOMEM;
 	}
 
-	knot_rrset_init(rrset, rowner, KNOT_RRTYPE_TXT, KNOT_CLASS_CH);
+	knot_rrset_init(rrset, rowner, KNOT_RRTYPE_TXT, KNOT_CLASS_CH, 0);
 	uint8_t rdata[response_len + 1];
 
 	rdata[0] = response_len;
 	memcpy(&rdata[1], response_str, response_len);
 
-	int ret = knot_rrset_add_rdata(rrset, rdata, response_len + 1, 0, mm);
+	int ret = knot_rrset_add_rdata(rrset, rdata, response_len + 1, mm);
 	if (ret != KNOT_EOK) {
-		knot_dname_free(&rrset->owner, mm);
+		knot_dname_free(rrset->owner, mm);
 		return ret;
 	}
 
