@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -67,29 +67,28 @@ static int dname_cname_synth(const knot_rrset_t *dname_rr,
 	if (owner_copy == NULL) {
 		return KNOT_ENOMEM;
 	}
-	knot_rrset_init(cname_rrset, owner_copy, KNOT_RRTYPE_CNAME, dname_rr->rclass);
+	knot_rrset_init(cname_rrset, owner_copy, KNOT_RRTYPE_CNAME, dname_rr->rclass,
+	                dname_rr->ttl);
 
 	/* Replace last labels of qname with DNAME. */
 	const knot_dname_t *dname_wire = dname_rr->owner;
-	const knot_dname_t *dname_tgt = knot_dname_target(&dname_rr->rrs);
-	int labels = knot_dname_labels(dname_wire, NULL);
-	knot_dname_t *cname = knot_dname_replace_suffix(qname, labels, dname_tgt);
+	const knot_dname_t *dname_tgt = knot_dname_target(dname_rr->rrs.rdata);
+	size_t labels = knot_dname_labels(dname_wire, NULL);
+	knot_dname_t *cname = knot_dname_replace_suffix(qname, labels, dname_tgt, mm);
 	if (cname == NULL) {
-		knot_dname_free(&owner_copy, mm);
+		knot_dname_free(owner_copy, mm);
 		return KNOT_ENOMEM;
 	}
 
 	/* Store DNAME into RDATA. */
-	int cname_size = knot_dname_size(cname);
+	size_t cname_size = knot_dname_size(cname);
 	uint8_t cname_rdata[cname_size];
 	memcpy(cname_rdata, cname, cname_size);
-	knot_dname_free(&cname, NULL);
+	knot_dname_free(cname, mm);
 
-	const knot_rdata_t *dname_data = knot_rdataset_at(&dname_rr->rrs, 0);
-	int ret = knot_rrset_add_rdata(cname_rrset, cname_rdata, cname_size,
-	                               knot_rdata_ttl(dname_data), mm);
+	int ret = knot_rrset_add_rdata(cname_rrset, cname_rdata, cname_size, mm);
 	if (ret != KNOT_EOK) {
-		knot_dname_free(&owner_copy, mm);
+		knot_dname_free(owner_copy, mm);
 		return ret;
 	}
 
@@ -103,10 +102,10 @@ static int dname_cname_synth(const knot_rrset_t *dname_rr,
 static bool dname_cname_cannot_synth(const knot_rrset_t *rrset, const knot_dname_t *qname)
 {
 	if (knot_dname_labels(qname, NULL) - knot_dname_labels(rrset->owner, NULL) +
-	    knot_dname_labels(knot_dname_target(&rrset->rrs), NULL) > KNOT_DNAME_MAXLABELS) {
+	    knot_dname_labels(knot_dname_target(rrset->rrs.rdata), NULL) > KNOT_DNAME_MAXLABELS) {
 		return true;
 	} else if (knot_dname_size(qname) - knot_dname_size(rrset->owner) +
-	           knot_dname_size(knot_dname_target(&rrset->rrs)) > KNOT_DNAME_MAXLEN) {
+	           knot_dname_size(knot_dname_target(rrset->rrs.rdata)) > KNOT_DNAME_MAXLEN) {
 		return true;
 	} else {
 		return false;
@@ -117,7 +116,7 @@ static bool dname_cname_cannot_synth(const knot_rrset_t *rrset, const knot_dname
 static bool have_dnssec(knotd_qdata_t *qdata)
 {
 	return knot_pkt_has_dnssec(qdata->query) &&
-	       zone_contents_is_signed(qdata->extra->zone->contents);
+	       qdata->extra->zone->contents->dnssec;
 }
 
 /*! \brief This is a wildcard-covered or any other terminal node for QNAME.
@@ -140,6 +139,7 @@ static int put_answer(knot_pkt_t *pkt, uint16_t type, knotd_qdata_t *qdata)
 
 	unsigned put_rr_flags = (qdata->params->flags & KNOTD_QUERY_FLAG_LIMIT_SIZE) ?
 	                        KNOT_PF_NULL : KNOT_PF_NOTRUNC;
+	put_rr_flags |= KNOT_PF_ORIGTTL;
 
 	int ret = KNOT_EOK;
 	switch (type) {
@@ -179,41 +179,10 @@ static int put_answer(knot_pkt_t *pkt, uint16_t type, knotd_qdata_t *qdata)
 static int put_authority_soa(knot_pkt_t *pkt, knotd_qdata_t *qdata,
                              const zone_contents_t *zone)
 {
-	knot_rrset_t soa_rrset = node_rrset(zone->apex, KNOT_RRTYPE_SOA);
+	knot_rrset_t soa = node_rrset(zone->apex, KNOT_RRTYPE_SOA);
 	knot_rrset_t rrsigs = node_rrset(zone->apex, KNOT_RRTYPE_RRSIG);
-
-	// if SOA's TTL is larger than MINIMUM, copy the RRSet and set
-	// MINIMUM as TTL
-	int ret = KNOT_EOK;
-	uint32_t flags = KNOT_PF_NOTRUNC;
-	uint32_t min = knot_soa_minimum(&soa_rrset.rrs);
-	const knot_rdata_t *soa_data = knot_rdataset_at(&soa_rrset.rrs, 0);
-	if (min < knot_rdata_ttl(soa_data)) {
-		knot_rrset_t copy;
-		knot_dname_t *dname_cpy = knot_dname_copy(soa_rrset.owner, &pkt->mm);
-		if (dname_cpy == NULL) {
-			return KNOT_ENOMEM;
-		}
-		knot_rrset_init(&copy, dname_cpy, soa_rrset.type, soa_rrset.rclass);
-		int ret = knot_rdataset_copy(&copy.rrs, &soa_rrset.rrs, &pkt->mm);
-		if (ret != KNOT_EOK) {
-			knot_dname_free(&dname_cpy, &pkt->mm);
-			return ret;
-		}
-		knot_rdata_t *copy_data = knot_rdataset_at(&copy.rrs, 0);
-		knot_rdata_set_ttl(copy_data, min);
-
-		flags |= KNOT_PF_FREE;
-		soa_rrset = copy;
-	}
-
-	ret = process_query_put_rr(pkt, qdata, &soa_rrset, &rrsigs,
-	                           KNOT_COMPR_HINT_NONE, flags);
-	if (ret != KNOT_EOK && (flags & KNOT_PF_FREE)) {
-		knot_rrset_clear(&soa_rrset, &pkt->mm);
-	}
-
-	return ret;
+	return process_query_put_rr(pkt, qdata, &soa, &rrsigs,
+	                            KNOT_COMPR_HINT_NONE, KNOT_PF_NOTRUNC);
 }
 
 /*! \brief Put the delegation NS RRSet to the Authority section. */
@@ -258,8 +227,8 @@ static int put_additional(knot_pkt_t *pkt, const knot_rrset_t *rr,
 			flags |= KNOT_PF_NOTRUNC;
 		}
 
-		uint16_t hint = knot_pkt_compr_hint(info, KNOT_COMPR_HINT_RDATA +
-		                                    glue->ns_pos);
+		uint16_t hint = knot_compr_hint(info, KNOT_COMPR_HINT_RDATA +
+		                                glue->ns_pos);
 		knot_rrset_t rrsigs = node_rrset(glue->node, KNOT_RRTYPE_RRSIG);
 		for (int k = 0; k < ar_type_count; ++k) {
 			knot_rrset_t rrset = node_rrset(glue->node, ar_type_list[k]);
@@ -341,7 +310,7 @@ static int follow_cname(knot_pkt_t *pkt, uint16_t rrtype, knotd_qdata_t *qdata)
 	}
 
 	/* Now follow the next CNAME TARGET. */
-	qdata->name = knot_cname_name(&cname_rr.rrs);
+	qdata->name = knot_cname_name(cname_rr.rrs.rdata);
 
 	return KNOTD_IN_STATE_FOLLOW;
 }
@@ -450,6 +419,11 @@ static int solve_name(int state, knot_pkt_t *pkt, knotd_qdata_t *qdata)
 
 static int solve_answer(int state, knot_pkt_t *pkt, knotd_qdata_t *qdata, void *ctx)
 {
+	/* Do not solve if already solved, e.g. in a module. */
+	if (state == KNOTD_IN_STATE_HIT) {
+		return state;
+	}
+
 	/* Get answer to QNAME. */
 	state = solve_name(state, pkt, qdata);
 
@@ -611,6 +585,13 @@ static int answer_query(knot_pkt_t *pkt, knotd_qdata_t *qdata)
 
 	bool with_dnssec = have_dnssec(qdata);
 
+	/* Resolve PREANSWER. */
+	if (plan != NULL) {
+		WALK_LIST(step, plan->stage[KNOTD_STAGE_PREANSWER]) {
+			SOLVE_STEP(step->process, state, step->ctx);
+		}
+	}
+
 	/* Resolve ANSWER. */
 	knot_pkt_begin(pkt, KNOT_ANSWER);
 	SOLVE_STEP(solve_answer, state, NULL);
@@ -668,7 +649,10 @@ int internet_process_query(knot_pkt_t *pkt, knotd_qdata_t *qdata)
 		NS_NEED_AUTH(qdata, qdata->extra->zone->name, ACL_ACTION_NONE);
 
 		/* Reserve space for TSIG. */
-		knot_pkt_reserve(pkt, knot_tsig_wire_size(&qdata->sign.tsig_key));
+		int ret = knot_pkt_reserve(pkt, knot_tsig_wire_size(&qdata->sign.tsig_key));
+		if (ret != KNOT_EOK) {
+			return KNOT_STATE_FAIL;
+		}
 	}
 
 	NS_NEED_ZONE_CONTENTS(qdata, KNOT_RCODE_SERVFAIL); /* Expired */
