@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 #include "knot/nameserver/log.h"
 #include "knot/nameserver/tsig_ctx.h"
 #include "knot/zone/serial.h"
-#include "dnssec/random.h"
+#include "libdnssec/random.h"
 #include "libknot/libknot.h"
 
 #define NOTIFY_LOG(priority, qdata, fmt...) \
@@ -59,7 +59,10 @@ int notify_process_query(knot_pkt_t *pkt, knotd_qdata_t *qdata)
 	}
 
 	/* Reserve space for TSIG. */
-	knot_pkt_reserve(pkt, knot_tsig_wire_size(&qdata->sign.tsig_key));
+	int ret = knot_pkt_reserve(pkt, knot_tsig_wire_size(&qdata->sign.tsig_key));
+	if (ret != KNOT_EOK) {
+		return KNOT_STATE_FAIL;
+	}
 
 	/* SOA RR in answer may be included, recover serial. */
 	zone_t *zone = (zone_t *)qdata->extra->zone;
@@ -67,7 +70,7 @@ int notify_process_query(knot_pkt_t *pkt, knotd_qdata_t *qdata)
 	if (answer->count > 0) {
 		const knot_rrset_t *soa = knot_pkt_rr(answer, 0);
 		if (soa->type == KNOT_RRTYPE_SOA) {
-			uint32_t serial = knot_soa_serial(&soa->rrs);
+			uint32_t serial = knot_soa_serial(soa->rrs.rdata);
 			uint32_t zone_serial = zone_contents_serial(zone->contents);
 			(void)zone_get_master_serial(zone, &zone_serial);
 			NOTIFY_LOG(LOG_INFO, qdata, "received, serial %u", serial);
