@@ -1,4 +1,4 @@
-/*  Copyright (C) 2015 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,19 +13,10 @@
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
-/*!
- * \file
- *
- * \brief NOTIFY request/reply API.
- *
- * \addtogroup query_processing
- * @{
- */
 
 #pragma once
 
 #include "libknot/packet/pkt.h"
-#include "knot/query/query.h"
 #include "knot/nameserver/process_query.h"
 
 #define NOTIFY_TIMEOUT 3 /*!< Interval between NOTIFY retries. */
@@ -36,6 +27,4 @@
  * \retval FAIL if it encountered an error.
  * \retval DONE if finished.
  */
-int notify_process_query(knot_pkt_t *pkt, struct query_data *qdata);
-
-/*! @} */
+int notify_process_query(knot_pkt_t *pkt, knotd_qdata_t *qdata);
