@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -114,21 +114,20 @@ static const zone_node_t *nsec3_encloser(const zone_node_t *closest)
  *
  * \return Next closer name, NULL on error.
  */
-static knot_dname_t *get_next_closer(const knot_dname_t *closest_encloser,
-                                     const knot_dname_t *name)
+static const knot_dname_t *get_next_closer(const knot_dname_t *closest_encloser,
+                                           const knot_dname_t *name)
 {
-	int ce_labels = knot_dname_labels(closest_encloser, NULL);
-	int qname_labels = knot_dname_labels(name, NULL);
-
-	// the common labels should match
-	assert(knot_dname_matched_labels(closest_encloser, name) == ce_labels);
-
-	// chop some labels from the qname
+	// make name only one label longer than closest_encloser
+	size_t ce_labels = knot_dname_labels(closest_encloser, NULL);
+	size_t qname_labels = knot_dname_labels(name, NULL);
 	for (int i = 0; i < (qname_labels - ce_labels - 1); ++i) {
 		name = knot_wire_next_label(name, NULL);
 	}
 
-	return knot_dname_copy(name, NULL);
+	// the common labels should match
+	assert(knot_dname_is_equal(knot_wire_next_label(name, NULL), closest_encloser));
+
+	return name;
 }
 
 /*!
@@ -244,16 +243,9 @@ static int put_nsec3_next_closer(const zone_node_t *cpe,
                                  knotd_qdata_t *qdata,
                                  knot_pkt_t *resp)
 {
-	knot_dname_t *next_closer = get_next_closer(cpe->owner, qname);
-	if (!next_closer) {
-		return KNOT_ENOMEM;
-	}
+	const knot_dname_t *next_closer = get_next_closer(cpe->owner, qname);
 
-	int ret = put_covering_nsec3(zone, next_closer, qdata, resp);
-
-	knot_dname_free(&next_closer, NULL);
-
-	return ret;
+	return put_covering_nsec3(zone, next_closer, qdata, resp);
 }
 
 /*!
@@ -363,19 +355,6 @@ static int put_wildcard_answer(const zone_node_t *wildcard,
 	return ret;
 }
 
-/*!
- * \brief Create a wildcard child of a name as a local variable.
- *
- * \param out     Name of the output wariable.
- * \param parent  Parent of the wildcard.
- */
-#define CREATE_WILDCARD(out, parent) \
-	int size = knot_dname_size(parent); \
-	if (size < 0 || size > KNOT_DNAME_MAXLEN - 2) return KNOT_EINVAL; \
-	uint8_t out[2 + size]; \
-	memcpy(out, "\x01""*", 2); \
-	memcpy(out + 2, parent, size);
-
 /*!
  * \brief Put NSECs for NXDOMAIN error into the response.
  *
@@ -414,7 +393,14 @@ static int put_nsec_nxdomain(const zone_contents_t *zone,
 
 	// NOTE: closest may be empty non-terminal and thus not authoritative.
 
-	CREATE_WILDCARD(wildcard, closest->owner)
+	size_t size = knot_dname_size(closest->owner);
+	if (size > KNOT_DNAME_MAXLEN - 2) {
+		return KNOT_EINVAL;
+	}
+	assert(size > 0);
+	uint8_t wildcard[2 + size];
+	memcpy(wildcard, "\x01""*", 2);
+	memcpy(wildcard + 2, closest->owner, size);
 
 	return put_covering_nsec(zone, wildcard, qdata, resp);
 }
@@ -453,9 +439,11 @@ static int put_nsec3_nxdomain(const knot_dname_t *qname,
 
 	// NSEC3 covering the (nonexistent) wildcard at the closest encloser.
 
-	CREATE_WILDCARD(wildcard, cpe->owner)
+	if (cpe->nsec3_wildcard_prev == NULL) {
+		return KNOT_ERROR;
+	}
 
-	return put_covering_nsec3(zone, wildcard, qdata, resp);
+	return put_nsec3_from_node(cpe->nsec3_wildcard_prev, qdata, resp);
 }
 
 /*!
@@ -477,15 +465,11 @@ static int put_nxdomain(const zone_contents_t *zone,
                         knotd_qdata_t *qdata,
                         knot_pkt_t *resp)
 {
-	int ret = 0;
-
 	if (knot_is_nsec3_enabled(zone)) {
-		ret = put_nsec3_nxdomain(qname, zone, closest, qdata, resp);
+		return put_nsec3_nxdomain(qname, zone, closest, qdata, resp);
 	} else {
-		ret = put_nsec_nxdomain(zone, previous, closest, qdata, resp);
+		return put_nsec_nxdomain(zone, previous, closest, qdata, resp);
 	}
-
-	return ret;
 }
 
 /*!
@@ -516,9 +500,9 @@ static int put_nsec_nodata(const zone_contents_t *zone,
 {
 	if (empty_nonterminal(match)) {
 		return put_nsec_nxdomain(zone, previous, closest, qdata, resp);
+	} else {
+		return put_nsec_from_node(match, qdata, resp);
 	}
-
-	return put_nsec_from_node(match, qdata, resp);
 }
 
 /*!
@@ -576,15 +560,11 @@ static int put_nodata(const zone_node_t *node,
                       knotd_qdata_t *qdata,
                       knot_pkt_t *resp)
 {
-	int ret = 0;
-
 	if (knot_is_nsec3_enabled(zone)) {
-		ret = put_nsec3_nodata(qname, zone, node, closest, qdata, resp);
+		return put_nsec3_nodata(qname, zone, node, closest, qdata, resp);
 	} else {
-		ret = put_nsec_nodata(zone, node, closest, previous, qdata, resp);
+		return put_nsec_nodata(zone, node, closest, previous, qdata, resp);
 	}
-
-	return ret;
 }
 
 int nsec_prove_wildcards(knot_pkt_t *pkt, knotd_qdata_t *qdata)
@@ -659,6 +639,7 @@ int nsec_append_rrsigs(knot_pkt_t *pkt, knotd_qdata_t *qdata, bool optional)
 	int ret = KNOT_EOK;
 	uint32_t flags = optional ? KNOT_PF_NOTRUNC : KNOT_PF_NULL;
 	flags |= KNOT_PF_FREE; // Free all RRSIGs, they are synthesized
+	flags |= KNOT_PF_ORIGTTL;
 
 	/* Append RRSIGs for section. */
 	struct rrsig_info *info = NULL;
