@@ -1,4 +1,4 @@
-/*  Copyright (C) 2015 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,33 +13,22 @@
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
-/*!
- * \file
- *
- * \brief DDNS UPDATE processing.
- *
- * \addtogroup query_processing
- * @{
- */
 
 #pragma once
 
 #include "libknot/packet/pkt.h"
+#include "knot/nameserver/process_query.h"
 #include "knot/zone/zone.h"
 
-struct query_data;
-
 /*!
  * \brief UPDATE query processing module.
  *
  * \return KNOT_STATE_* processing states
  */
-int update_process_query(knot_pkt_t *pkt, struct query_data *qdata);
+int update_process_query(knot_pkt_t *pkt, knotd_qdata_t *qdata);
 
 /*!
  * \brief Processes serialized packet with DDNS. Function expects that the
  *        query is already authenticated and TSIG signature is verified.
  */
 void updates_execute(conf_t *conf, zone_t *zone);
-
-/*! @} */
