@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,17 +25,13 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <urcu.h>
-#ifdef HAVE_SYS_UIO_H			// struct iovec (OpenBSD)
+#ifdef HAVE_SYS_UIO_H	// struct iovec (OpenBSD)
 #include <sys/uio.h>
 #endif // HAVE_SYS_UIO_H
-#ifdef HAVE_CAP_NG_H
-#include <cap-ng.h>
-#endif /* HAVE_CAP_NG_H */
 
-#include "dnssec/random.h"
+#include "libdnssec/random.h"
 #include "knot/server/server.h"
 #include "knot/server/tcp-handler.h"
-#include "knot/common/fdset.h"
 #include "knot/common/log.h"
 #include "knot/nameserver/process_query.h"
 #include "knot/query/layer.h"
@@ -58,11 +54,9 @@ typedef struct tcp_context {
 	unsigned thread_id;              /*!< Thread identifier. */
 } tcp_context_t;
 
-/*
- * Forward decls.
- */
-#define TCP_THROTTLE_LO 0 /*!< Minimum recovery time on errors. */
-#define TCP_THROTTLE_HI 2 /*!< Maximum recovery time on errors. */
+#define TCP_SWEEP_INTERVAL 2 /*!< [secs] granularity of connection sweeping. */
+#define TCP_THROTTLE_LO    0 /*!< Minimum recovery time on errors. */
+#define TCP_THROTTLE_HI    2 /*!< Maximum recovery time on errors. */
 
 /*! \brief Calculate TCP throttle time (random). */
 static inline int tcp_throttle(void) {
@@ -100,14 +94,17 @@ static bool tcp_send_state(int state)
 	return (state != KNOT_STATE_FAIL && state != KNOT_STATE_NOOP);
 }
 
-/*!
- * \brief TCP event handler function.
- */
 static int tcp_handle(tcp_context_t *tcp, int fd,
                       struct iovec *rx, struct iovec *tx)
 {
+	/* Get peer name. */
+	struct sockaddr_storage ss;
+	socklen_t addrlen = sizeof(struct sockaddr_storage);
+	if (getpeername(fd, (struct sockaddr *)&ss, &addrlen) != 0) {
+		return KNOT_EINVAL;
+	}
+
 	/* Create query processing parameter. */
-	struct sockaddr_storage ss = { 0 };
 	knotd_qdata_params_t params = {
 		.remote = &ss,
 		.socket = fd,
@@ -118,12 +115,6 @@ static int tcp_handle(tcp_context_t *tcp, int fd,
 	rx->iov_len = KNOT_WIRE_MAX_PKTSIZE;
 	tx->iov_len = KNOT_WIRE_MAX_PKTSIZE;
 
-	/* Receive peer name. */
-	socklen_t addrlen = sizeof(struct sockaddr_storage);
-	if (getpeername(fd, (struct sockaddr *)&ss, &addrlen) < 0) {
-		;
-	}
-
 	/* Timeout. */
 	rcu_read_lock();
 	int timeout = 1000 * conf()->cache.srv_tcp_reply_timeout;
@@ -170,14 +161,13 @@ static int tcp_handle(tcp_context_t *tcp, int fd,
 	/* Reset after processing. */
 	knot_layer_finish(&tcp->layer);
 
-	/* Cleanup. */
-	knot_pkt_free(&query);
-	knot_pkt_free(&ans);
+	/* Flush per-query memory (including query and answer packets). */
+	mp_flush(tcp->layer.mm->ctx);
 
 	return ret;
 }
 
-int tcp_accept(int fd)
+static int tcp_accept(int fd)
 {
 	/* Accept incoming connection. */
 	int incoming = net_accept(fd, NULL);
@@ -229,10 +219,6 @@ static int tcp_event_serve(tcp_context_t *tcp, unsigned i)
 {
 	int fd = tcp->set.pfd[i].fd;
 	int ret = tcp_handle(tcp, fd, &tcp->iov[0], &tcp->iov[1]);
-
-	/* Flush per-query memory. */
-	mp_flush(tcp->layer.mm->ctx);
-
 	if (ret == KNOT_EOK) {
 		/* Update socket activity timer. */
 		rcu_read_lock();
@@ -301,7 +287,7 @@ static int tcp_wait_for_events(tcp_context_t *tcp)
 
 int tcp_master(dthread_t *thread)
 {
-	if (!thread || !thread->data) {
+	if (thread == NULL || thread->data == NULL) {
 		return KNOT_EINVAL;
 	}
 
@@ -310,16 +296,16 @@ int tcp_master(dthread_t *thread)
 
 	int ret = KNOT_EOK;
 	ref_t *ref = NULL;
-	tcp_context_t tcp;
-	memset(&tcp, 0, sizeof(tcp_context_t));
 
 	/* Create big enough memory cushion. */
-	knot_mm_t mm = { 0 };
+	knot_mm_t mm;
 	mm_ctx_mempool(&mm, 16 * MM_DEFAULT_BLKSIZE);
 
 	/* Create TCP answering context. */
-	tcp.server = handler->server;
-	tcp.thread_id = handler->thread_id[dt_get_id(thread)];
+	tcp_context_t tcp = {
+		.server = handler->server,
+		.thread_id = handler->thread_id[dt_get_id(thread)]
+	};
 	knot_layer_init(&tcp.layer, &mm, process_query_layer());
 
 	/* Prepare structures for bound sockets. */
