@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -76,10 +76,10 @@ static int replace_rdataset_with_copy(zone_node_t *node, uint16_t type)
 		return KNOT_ENOMEM;
 	}
 
-	memcpy(copy, rrs->data, knot_rdataset_size(rrs));
+	memcpy(copy, rrs->rdata, knot_rdataset_size(rrs));
 
 	// Store new data into node RRS.
-	rrs->data = copy;
+	rrs->rdata = copy;
 
 	return KNOT_EOK;
 }
@@ -114,25 +114,25 @@ static int add_new_data(apply_ctx_t *ctx, knot_rdata_t *new_data)
 }
 
 /*! \brief Returns true if given RR is present in node and can be removed. */
-static bool can_remove(const zone_node_t *node, const knot_rrset_t *rr)
+static bool can_remove(const zone_node_t *node, const knot_rrset_t *rrset)
 {
 	if (node == NULL) {
 		// Node does not exist, cannot remove anything.
 		return false;
 	}
-	const knot_rdataset_t *node_rrs = node_rdataset(node, rr->type);
+	const knot_rdataset_t *node_rrs = node_rdataset(node, rrset->type);
 	if (node_rrs == NULL) {
 		// Node does not have this type at all.
 		return false;
 	}
 
-	const bool compare_ttls = false;
-	for (uint16_t i = 0; i < rr->rrs.rr_count; ++i) {
-		knot_rdata_t *rr_cmp = knot_rdataset_at(&rr->rrs, i);
-		if (knot_rdataset_member(node_rrs, rr_cmp, compare_ttls)) {
+	knot_rdata_t *rr_cmp = rrset->rrs.rdata;
+	for (uint16_t i = 0; i < rrset->rrs.count; ++i) {
+		if (knot_rdataset_member(node_rrs, rr_cmp)) {
 			// At least one RR matches.
 			return true;
 		}
+		rr_cmp = knot_rdataset_next(rr_cmp);
 	}
 
 	// Node does have the type, but no RRs match.
@@ -191,11 +191,15 @@ static int apply_single(apply_ctx_t *ctx, const changeset_t *chset)
 
 	zone_contents_t *contents = ctx->contents;
 
-	bool ignore_soa = (chset->soa_from == NULL && chset->soa_to == NULL);
-
 	// check if serial matches
 	const knot_rdataset_t *soa = node_rdataset(contents->apex, KNOT_RRTYPE_SOA);
-	if (soa == NULL || (!ignore_soa && knot_soa_serial(soa) != knot_soa_serial(&chset->soa_from->rrs))) {
+
+	// either we are in the mode of ignoring SOA (both NULL), or we shall be strict and apply SOA later
+	bool ignore_soa = (chset->soa_from == NULL && chset->soa_to == NULL);
+	bool soa_mismatch = (chset->soa_from == NULL || chset->soa_to == NULL || soa == NULL ||
+			     knot_soa_serial(soa->rdata) != knot_soa_serial(chset->soa_from->rrs.rdata));
+
+	if (soa == NULL || (!ignore_soa && soa_mismatch)) {
 		return KNOT_EINVAL;
 	}
 
@@ -265,7 +269,7 @@ int apply_add_rr(apply_ctx_t *ctx, const knot_rrset_t *rr)
 	knot_rrset_t changed_rrset = node_rrset(node, rr->type);
 	if (!knot_rrset_empty(&changed_rrset)) {
 		// Modifying existing RRSet.
-		knot_rdata_t *old_data = changed_rrset.rrs.data;
+		knot_rdata_t *old_data = changed_rrset.rrs.rdata;
 		int ret = replace_rdataset_with_copy(node, rr->type);
 		if (ret != KNOT_EOK) {
 			return ret;
@@ -284,16 +288,23 @@ int apply_add_rr(apply_ctx_t *ctx, const knot_rrset_t *rr)
 	if (ret == KNOT_EOK || ret == KNOT_ETTL) {
 		// RR added, store for possible rollback.
 		knot_rdataset_t *rrs = node_rdataset(node, rr->type);
-		int data_ret = add_new_data(ctx, rrs->data);
+		int data_ret = add_new_data(ctx, rrs->rdata);
 		if (data_ret != KNOT_EOK) {
 			knot_rdataset_clear(rrs, NULL);
 			return data_ret;
 		}
 
 		if (ret == KNOT_ETTL) {
+			char buff[KNOT_DNAME_TXT_MAXLEN + 1];
+			char *owner = knot_dname_to_str(buff, rr->owner, sizeof(buff));
+			if (owner == NULL) {
+				owner = "";
+			}
+			char type[16] = { '\0' };
+			knot_rrtype_to_string(rr->type, type, sizeof(type));
 			log_zone_notice(contents->apex->owner,
-			                "rrset (type %u) TTL mismatch, updated to %u",
-			                rr->type, knot_rrset_ttl(rr));
+			                "TTL mismatch, owner %s, type %s, "
+			                "TTL set to %u", owner, type, rr->ttl);
 			return KNOT_EOK;
 		}
 	}
@@ -320,7 +331,7 @@ int apply_remove_rr(apply_ctx_t *ctx, const knot_rrset_t *rr)
 	                    contents->nsec3_nodes : contents->nodes;
 
 	knot_rrset_t removed_rrset = node_rrset(node, rr->type);
-	knot_rdata_t *old_data = removed_rrset.rrs.data;
+	knot_rdata_t *old_data = removed_rrset.rrs.rdata;
 	int ret = replace_rdataset_with_copy(node, rr->type);
 	if (ret != KNOT_EOK) {
 		return ret;
@@ -335,25 +346,25 @@ int apply_remove_rr(apply_ctx_t *ctx, const knot_rrset_t *rr)
 
 	knot_rdataset_t *changed_rrs = node_rdataset(node, rr->type);
 	// Subtract changeset RRS from node RRS.
-	ret = knot_rdataset_subtract(changed_rrs, &rr->rrs, false, NULL);
+	ret = knot_rdataset_subtract(changed_rrs, &rr->rrs, NULL);
 	if (ret != KNOT_EOK) {
 		clear_new_rrs(node, rr->type);
 		return ret;
 	}
 
-	if (changed_rrs->rr_count > 0) {
+	if (changed_rrs->count > 0) {
 		// Subtraction left some data in RRSet, store it for rollback.
-		ret = add_new_data(ctx, changed_rrs->data);
+		ret = add_new_data(ctx, changed_rrs->rdata);
 		if (ret != KNOT_EOK) {
 			knot_rdataset_clear(changed_rrs, NULL);
 			return ret;
 		}
 	} else {
-		// RRSet is empty now, remove it from node, all data freed.
+		// RRSet is empty now, remove it from node, all data freed, except additionals.
 		node_remove_rdataset(node, rr->type);
 		// If node is empty now, delete it from zone tree.
 		if (node->rrset_count == 0 && node != contents->apex) {
-			zone_tree_delete_empty_node(tree, node);
+			zone_tree_delete_empty(tree, node);
 		}
 	}
 
@@ -387,80 +398,6 @@ int apply_prepare_to_sign(apply_ctx_t *ctx)
 	return zone_contents_adjust_pointers(ctx->contents);
 }
 
-int apply_changesets(apply_ctx_t *ctx, zone_contents_t *old_contents,
-                     list_t *chsets, zone_contents_t **new_contents)
-{
-	if (ctx == NULL || old_contents == NULL || chsets == NULL ||
-	    EMPTY_LIST(*chsets) || new_contents == NULL) {
-		return KNOT_EINVAL;
-	}
-
-	zone_contents_t *contents_copy = NULL;
-	int ret = apply_prepare_zone_copy(old_contents, &contents_copy);
-	if (ret != KNOT_EOK) {
-		return ret;
-	}
-
-	ctx->contents = contents_copy;
-
-	changeset_t *set = NULL;
-	WALK_LIST(set, *chsets) {
-		ret = apply_single(ctx, set);
-		if (ret != KNOT_EOK) {
-			update_rollback(ctx);
-			update_free_zone(&ctx->contents);
-			return ret;
-		}
-	}
-
-	assert(contents_copy->apex != NULL);
-
-	ret = zone_contents_adjust_full(contents_copy);
-	if (ret != KNOT_EOK) {
-		update_rollback(ctx);
-		update_free_zone(&ctx->contents);
-		return ret;
-	}
-
-	*new_contents = contents_copy;
-
-	return KNOT_EOK;
-}
-
-int apply_changeset(apply_ctx_t *ctx, zone_contents_t *old_contents,
-                    changeset_t *ch, zone_contents_t **new_contents)
-{
-	if (ctx == NULL || old_contents == NULL || ch == NULL || new_contents == NULL) {
-		return KNOT_EINVAL;
-	}
-
-	zone_contents_t *contents_copy = NULL;
-	int ret = apply_prepare_zone_copy(old_contents, &contents_copy);
-	if (ret != KNOT_EOK) {
-		return ret;
-	}
-
-	ctx->contents = contents_copy;
-
-	ret = apply_single(ctx, ch);
-	if (ret != KNOT_EOK) {
-		update_rollback(ctx);
-		update_free_zone(&ctx->contents);
-		return ret;
-	}
-
-	ret = zone_contents_adjust_full(contents_copy);
-	if (ret != KNOT_EOK) {
-		update_rollback(ctx);
-		update_free_zone(&ctx->contents);
-		return ret;
-	}
-
-	*new_contents = contents_copy;
-
-	return KNOT_EOK;
-}
-
 int apply_changesets_directly(apply_ctx_t *ctx, list_t *chsets)
 {
 	if (ctx == NULL || ctx->contents == NULL || chsets == NULL) {
@@ -532,18 +469,17 @@ void update_rollback(apply_ctx_t *ctx)
 	init_list(&ctx->old_data);
 }
 
-void update_free_zone(zone_contents_t **contents)
+void update_free_zone(zone_contents_t *contents)
 {
-	if (contents == NULL || *contents == NULL) {
+	if (contents == NULL) {
 		return;
 	}
 
-	zone_tree_apply((*contents)->nodes, free_additional, NULL);
-	zone_tree_deep_free(&(*contents)->nodes);
-	zone_tree_deep_free(&(*contents)->nsec3_nodes);
+	(void)zone_tree_apply(contents->nodes, free_additional, NULL);
+	zone_tree_deep_free(&contents->nodes);
+	zone_tree_deep_free(&contents->nsec3_nodes);
 
-	dnssec_nsec3_params_free(&(*contents)->nsec3_params);
+	dnssec_nsec3_params_free(&contents->nsec3_params);
 
-	free(*contents);
-	*contents = NULL;
+	free(contents);
 }
