@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@ static int handle_soa(knot_rrset_t **soa, const knot_rrset_t *rrset)
 	assert(rrset);
 
 	if (*soa != NULL) {
-		knot_rrset_free(soa, NULL);
+		knot_rrset_free(*soa, NULL);
 	}
 
 	*soa = knot_rrset_copy(rrset, NULL);
@@ -48,7 +48,9 @@ static int add_rr_to_contents(zone_contents_t *z, const knot_rrset_t *rrset)
 	zone_node_t *n = NULL;
 	int ret = zone_contents_add_rr(z, rrset, &n);
 	UNUSED(n);
-	return ret;
+
+	// We don't care of TTLs.
+	return ret == KNOT_ETTL ? KNOT_EOK : ret;
 }
 
 /*! \brief Cleans up trie iterations. */
@@ -162,17 +164,20 @@ static void check_redundancy(zone_contents_t *counterpart, const knot_rrset_t *r
 
 	// Subtract the data from node's RRSet.
 	knot_rdataset_t *rrs = node_rdataset(node, rr->type);
+	uint32_t rrs_ttl = node_rrset(node, rr->type).ttl;
 
-	if (fixed_rr != NULL && *fixed_rr != NULL) {
-		int ret = knot_rdataset_subtract(&(*fixed_rr)->rrs, rrs, true, NULL);
+	if (fixed_rr != NULL && *fixed_rr != NULL && (*fixed_rr)->ttl == rrs_ttl) {
+		int ret = knot_rdataset_subtract(&(*fixed_rr)->rrs, rrs, NULL);
 		if (ret != KNOT_EOK) {
 			return;
 		}
 	}
 
-	int ret = knot_rdataset_subtract(rrs, &rr->rrs, true, NULL);
-	if (ret != KNOT_EOK) {
-		return;
+	if (rr->ttl == rrs_ttl) {
+		int ret = knot_rdataset_subtract(rrs, &rr->rrs, NULL);
+		if (ret != KNOT_EOK) {
+			return;
+		}
 	}
 
 	if (knot_rdataset_size(rrs) == 0) {
@@ -182,8 +187,8 @@ static void check_redundancy(zone_contents_t *counterpart, const knot_rrset_t *r
 		if (node->rrset_count == 0 && node != counterpart->apex) {
 			// Remove empty node.
 			zone_tree_t *t = knot_rrset_is_nsec3rel(rr) ?
-								 counterpart->nsec3_nodes : counterpart->nodes;
-			zone_tree_delete_empty_node(t, node);
+			                 counterpart->nsec3_nodes : counterpart->nodes;
+			zone_tree_delete_empty(t, node);
 		}
 	}
 
@@ -201,7 +206,7 @@ int changeset_init(changeset_t *ch, const knot_dname_t *apex)
 	}
 	ch->remove = zone_contents_new(apex);
 	if (ch->remove == NULL) {
-		zone_contents_free(&ch->add);
+		zone_contents_free(ch->add);
 		return KNOT_ENOMEM;
 	}
 
@@ -299,9 +304,9 @@ int changeset_add_addition(changeset_t *ch, const knot_rrset_t *rrset, changeset
 	int ret = knot_rrset_empty(to_add) ? KNOT_EOK : add_rr_to_contents(ch->add, to_add);
 
 	if (flags & CHANGESET_CHECK) {
-		knot_rrset_free((knot_rrset_t **)&rrset, NULL);
+		knot_rrset_free((knot_rrset_t *)rrset, NULL);
 	}
-	knot_rrset_free(&rrset_cancelout, NULL);
+	knot_rrset_free(rrset_cancelout, NULL);
 
 	return ret;
 }
@@ -336,14 +341,11 @@ int changeset_add_removal(changeset_t *ch, const knot_rrset_t *rrset, changeset_
 	int ret = knot_rrset_empty(to_remove) ? KNOT_EOK : add_rr_to_contents(ch->remove, to_remove);
 
 	if (flags & CHANGESET_CHECK) {
-		knot_rrset_free((knot_rrset_t **)&rrset, NULL);
+		knot_rrset_free((knot_rrset_t *)rrset, NULL);
 	}
+	knot_rrset_free(rrset_cancelout, NULL);
 
-	knot_rrset_free(&rrset_cancelout, NULL);
-
-	// we don't care of TTLs at removals anyway (updates/apply.c/can_remove()/compare_ttls)
-	// in practice, this happens at merging changesets
-	return (ret == KNOT_ETTL ? KNOT_EOK : ret);
+	return ret;
 }
 
 int changeset_remove_addition(changeset_t *ch, const knot_rrset_t *rrset)
@@ -351,7 +353,7 @@ int changeset_remove_addition(changeset_t *ch, const knot_rrset_t *rrset)
 	if (rrset->type == KNOT_RRTYPE_SOA) {
 		/* Do not add SOAs into actual contents. */
 		if (ch->soa_to != NULL) {
-			knot_rrset_free(&ch->soa_to, NULL);
+			knot_rrset_free(ch->soa_to, NULL);
 			ch->soa_to = NULL;
 		}
 		return KNOT_EOK;
@@ -366,7 +368,7 @@ int changeset_remove_removal(changeset_t *ch, const knot_rrset_t *rrset)
 	if (rrset->type == KNOT_RRTYPE_SOA) {
 		/* Do not add SOAs into actual contents. */
 		if (ch->soa_from != NULL) {
-			knot_rrset_free(&ch->soa_from, NULL);
+			knot_rrset_free(ch->soa_from, NULL);
 			ch->soa_from = NULL;
 		}
 		return KNOT_EOK;
@@ -415,7 +417,7 @@ int changeset_merge(changeset_t *ch1, const changeset_t *ch2, int flags)
 	if (soa_copy == NULL && ch2->soa_to) {
 		return KNOT_ENOMEM;
 	}
-	knot_rrset_free(&ch1->soa_to, NULL);
+	knot_rrset_free(ch1->soa_to, NULL);
 	ch1->soa_to = soa_copy;
 
 	return KNOT_EOK;
@@ -435,21 +437,25 @@ static int preapply_fix_rrset(const knot_rrset_t *apply, bool adding, void *data
 	if (adding && zrdataset == NULL) {
 		return KNOT_EOK;
 	}
+
 	knot_rrset_t *fixrrset;
-	int ret = KNOT_EOK;
 	if (adding) {
-		fixrrset = knot_rrset_new(apply->owner, apply->type, apply->rclass, ctx->mm);
+		fixrrset = knot_rrset_new(apply->owner, apply->type, apply->rclass,
+		                          apply->ttl, ctx->mm);
 	} else {
 		fixrrset = knot_rrset_copy(apply, ctx->mm);
 	}
 	if (fixrrset == NULL) {
 		return KNOT_ENOMEM;
 	}
+
+	int ret = KNOT_EOK;
 	if (adding) {
 		ret = knot_rdataset_intersect(zrdataset, &apply->rrs, &fixrrset->rrs, ctx->mm);
 	} else {
-		if (zrdataset != NULL) {
-			ret = knot_rdataset_subtract(&fixrrset->rrs, zrdataset, true, ctx->mm);
+		uint32_t zrrset_ttl = node_rrset(znode, apply->type).ttl;
+		if (zrdataset != NULL && fixrrset->ttl == zrrset_ttl) {
+			ret = knot_rdataset_subtract(&fixrrset->rrs, zrdataset, ctx->mm);
 		}
 	}
 	if (ret == KNOT_EOK && !knot_rrset_empty(fixrrset)) {
@@ -459,7 +465,8 @@ static int preapply_fix_rrset(const knot_rrset_t *apply, bool adding, void *data
 			ret = changeset_add_addition(ctx->fixing, fixrrset, 0);
 		}
 	}
-	knot_rrset_free(&fixrrset, ctx->mm);
+
+	knot_rrset_free(fixrrset, ctx->mm);
 	return ret;
 }
 
@@ -524,6 +531,34 @@ int changeset_cancelout(changeset_t *ch)
 	return ret;
 }
 
+bool changeset_differs_just_serial(const changeset_t *ch)
+{
+	if (ch == NULL || ch->soa_from == NULL || ch->soa_to == NULL) {
+		return false;
+	}
+
+	knot_rrset_t *soa_to_cpy = knot_rrset_copy(ch->soa_to, NULL);
+	knot_soa_serial_set(soa_to_cpy->rrs.rdata, knot_soa_serial(ch->soa_from->rrs.rdata));
+
+	bool ret = knot_rrset_equal(ch->soa_from, soa_to_cpy, KNOT_RRSET_COMPARE_WHOLE);
+	knot_rrset_free(soa_to_cpy, NULL);
+
+	changeset_iter_t itt;
+	changeset_iter_all(&itt, ch);
+
+	knot_rrset_t rrset = changeset_iter_next(&itt);
+	while (!knot_rrset_empty(&rrset) && ret) {
+		if (rrset.type != KNOT_RRTYPE_RRSIG || rrset.rrs.count != 1 ||
+		    knot_rrsig_type_covered(rrset.rrs.rdata) != KNOT_RRTYPE_SOA) {
+			ret = false;
+		}
+		rrset = changeset_iter_next(&itt);
+	}
+	changeset_iter_clear(&itt);
+
+	return ret;
+}
+
 int changeset_to_contents(changeset_t *ch, zone_contents_t **out)
 {
 	assert(ch->soa_from == NULL);
@@ -532,12 +567,12 @@ int changeset_to_contents(changeset_t *ch, zone_contents_t **out)
 
 	*out = ch->add;
 	int ret = add_rr_to_contents(*out, ch->soa_to);
-	knot_rrset_free(&ch->soa_to, NULL);
+	knot_rrset_free(ch->soa_to, NULL);
 	if (ret != KNOT_EOK) {
-		zone_contents_deep_free(out);
+		zone_contents_deep_free(*out);
 	}
 
-	zone_contents_deep_free(&ch->remove);
+	zone_contents_deep_free(ch->remove);
 	free(ch->data);
 	free(ch);
 	return ret;
@@ -557,7 +592,7 @@ changeset_t *changeset_from_contents(const zone_contents_t *contents)
 
 	node_remove_rdataset(copy->apex, KNOT_RRTYPE_SOA);
 
-	zone_contents_deep_free(&res->add);
+	zone_contents_deep_free(res->add);
 	res->add = copy;
 	return res;
 }
@@ -568,11 +603,11 @@ void changeset_from_contents_free(changeset_t *ch)
 	assert(ch->soa_from == NULL);
 	assert(zone_contents_is_empty(ch->remove));
 
-	update_free_zone(&ch->add);
+	update_free_zone(ch->add);
 
-	zone_contents_deep_free(&ch->remove);
-	knot_rrset_free(&ch->soa_from, NULL);
-	knot_rrset_free(&ch->soa_to, NULL);
+	zone_contents_deep_free(ch->remove);
+	knot_rrset_free(ch->soa_from, NULL);
+	knot_rrset_free(ch->soa_to, NULL);
 	free(ch->data);
 	free(ch);
 }
@@ -608,16 +643,63 @@ void changeset_clear(changeset_t *ch)
 	}
 
 	// Delete RRSets in lists, in case there are any left
-	zone_contents_deep_free(&ch->add);
-	zone_contents_deep_free(&ch->remove);
+	zone_contents_deep_free(ch->add);
+	zone_contents_deep_free(ch->remove);
+	ch->add = NULL;
+	ch->remove = NULL;
 
-	knot_rrset_free(&ch->soa_from, NULL);
-	knot_rrset_free(&ch->soa_to, NULL);
+	knot_rrset_free(ch->soa_from, NULL);
+	knot_rrset_free(ch->soa_to, NULL);
+	ch->soa_from = NULL;
+	ch->soa_to = NULL;
 
 	// Delete binary data
 	free(ch->data);
 }
 
+changeset_t *changeset_clone(const changeset_t *ch)
+{
+	if (ch == NULL) {
+		return NULL;
+	}
+
+	changeset_t *res = changeset_new(ch->add->apex->owner);
+	if (res == NULL) {
+		return NULL;
+	}
+
+	res->soa_from = knot_rrset_copy(ch->soa_from, NULL);
+	res->soa_to = knot_rrset_copy(ch->soa_to, NULL);
+
+	int ret = KNOT_EOK;
+	changeset_iter_t itt;
+
+	changeset_iter_rem(&itt, ch);
+	knot_rrset_t rr = changeset_iter_next(&itt);
+	while (!knot_rrset_empty(&rr) && ret == KNOT_EOK) {
+		ret = changeset_add_removal(res, &rr, 0);
+		rr = changeset_iter_next(&itt);
+	}
+	changeset_iter_clear(&itt);
+
+	changeset_iter_add(&itt, ch);
+	rr = changeset_iter_next(&itt);
+	while (!knot_rrset_empty(&rr) && ret == KNOT_EOK) {
+		ret = changeset_add_addition(res, &rr, 0);
+		rr = changeset_iter_next(&itt);
+	}
+	changeset_iter_clear(&itt);
+
+	if ((ch->soa_from != NULL && res->soa_from == NULL) ||
+	    (ch->soa_to != NULL && res->soa_to == NULL) ||
+	    ret != KNOT_EOK) {
+		changeset_free(res);
+		return NULL;
+	}
+
+	return res;
+}
+
 void changeset_free(changeset_t *ch)
 {
 	changeset_clear(ch);
