@@ -1,4 +1,4 @@
-/*  Copyright (C) 2015 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -33,7 +33,7 @@ static void rrset_list_clear(list_t *l)
 	WALK_LIST_DELSAFE(n, nxt, *l) {
 		ptrnode_t *ptr_n = (ptrnode_t *)n;
 		knot_rrset_t *rrset = (knot_rrset_t *)ptr_n->d;
-		knot_rrset_free(&rrset, NULL);
+		knot_rrset_free(rrset, NULL);
 		free(n);
 	};
 }
@@ -160,18 +160,17 @@ static int check_not_in_use(zone_update_t *update,
 	}
 }
 
-/*!< \brief Returns true if rrset has 0 data or RDATA of size 0 (we need TTL).*/
+/*!< \brief Returns true if rrset has 0 data or RDATA of size 0 (we need TTL). */
 static bool rrset_empty(const knot_rrset_t *rrset)
 {
-	uint16_t rr_count = rrset->rrs.rr_count;
-	if (rr_count == 0) {
+	switch (rrset->rrs.count) {
+	case 0:
 		return true;
+	case 1:
+		return rrset->rrs.rdata->len == 0;
+	default:
+		return false;
 	}
-	if (rr_count == 1) {
-		const knot_rdata_t *rr = knot_rdataset_at(&rrset->rrs, 0);
-		return knot_rdata_rdlen(rr) == 0;
-	}
-	return false;
 }
 
 /*< \brief Returns true if DDNS should deny updating DNSSEC-related record. */
@@ -197,12 +196,12 @@ static int process_prereq(const knot_rrset_t *rrset, uint16_t qclass,
                           zone_update_t *update, uint16_t *rcode,
                           list_t *rrset_list)
 {
-	if (knot_rdata_ttl(knot_rdataset_at(&rrset->rrs, 0)) != 0) {
+	if (rrset->ttl != 0) {
 		*rcode = KNOT_RCODE_FORMERR;
 		return KNOT_EMALF;
 	}
 
-	if (!knot_dname_in(update->zone->name, rrset->owner)) {
+	if (knot_dname_in_bailiwick(rrset->owner, update->zone->name) < 0) {
 		*rcode = KNOT_RCODE_NOTZONE;
 		return KNOT_EOUTOFZONE;
 	}
@@ -278,15 +277,12 @@ static bool should_replace(const knot_rrset_t *rrset)
 
 /*!< \brief Returns true if node contains given RR in its RRSets. */
 static bool node_contains_rr(const zone_node_t *node,
-                             const knot_rrset_t *rr)
+                             const knot_rrset_t *rrset)
 {
-	const knot_rdataset_t *zone_rrs = node_rdataset(node, rr->type);
-	if (zone_rrs) {
-		assert(rr->rrs.rr_count == 1);
-		const bool compare_ttls = false;
-		return knot_rdataset_member(zone_rrs,
-		                            knot_rdataset_at(&rr->rrs, 0),
-		                            compare_ttls);
+	const knot_rdataset_t *zone_rrs = node_rdataset(node, rrset->type);
+	if (zone_rrs != NULL) {
+		assert(rrset->rrs.count == 1);
+		return knot_rdataset_member(zone_rrs, rrset->rrs.rdata);
 	} else {
 		return false;
 	}
@@ -316,7 +312,7 @@ static bool skip_soa(const knot_rrset_t *rr, int64_t sn)
 {
 	if (rr->type == KNOT_RRTYPE_SOA &&
 	    (rr->rclass == KNOT_CLASS_NONE || rr->rclass == KNOT_CLASS_ANY ||
-	     (serial_compare(knot_soa_serial(&rr->rrs), sn) != SERIAL_GREATER))) {
+	     (serial_compare(knot_soa_serial(rr->rrs.rdata), sn) != SERIAL_GREATER))) {
 		return true;
 	}
 
@@ -497,7 +493,7 @@ static int process_rem_rr(const knot_rrset_t *rr,
 			// Zone without apex NS.
 			return KNOT_EOK;
 		}
-		if (ns_rrs->rr_count == 1) {
+		if (ns_rrs->count == 1) {
 			// Cannot remove last apex NS RR.
 			return KNOT_EOK;
 		}
@@ -510,7 +506,7 @@ static int process_rem_rr(const knot_rrset_t *rr,
 	}
 
 	knot_rdataset_t *rrs = node_rdataset(node, rr->type);
-	if (!knot_rdataset_member(rrs, rr->rrs.data, false)) {
+	if (!knot_rdataset_member(rrs, rr->rrs.rdata)) {
 		// Node does not contain this RR
 		return KNOT_EOK;
 	}
@@ -568,12 +564,12 @@ static int process_rem_node(const knot_rrset_t *rr,
 		knot_rrset_t rrset = node_rrset_at(node_copy, rrset_count - i - 1);
 		int ret = process_rem_rrset(&rrset, node_copy, update);
 		if (ret != KNOT_EOK) {
-			node_free(&node_copy, NULL);
+			node_free(node_copy, NULL);
 			return ret;
 		}
 	}
 
-	node_free(&node_copy, NULL);
+	node_free(node_copy, NULL);
 
 	return KNOT_EOK;
 }
@@ -632,12 +628,12 @@ static int check_update(const knot_rrset_t *rrset, const knot_pkt_t *query,
 	/* Accept both subdomain and dname match. */
 	const knot_dname_t *owner = rrset->owner;
 	const knot_dname_t *qname = knot_pkt_qname(query);
-	const bool is_sub = knot_dname_is_sub(owner, qname);
-	const bool is_apex = knot_dname_is_equal(owner, qname);
-	if (!is_sub && !is_apex) {
+	const int in_bailiwick = knot_dname_in_bailiwick(owner, qname);
+	if (in_bailiwick < 0) {
 		*rcode = KNOT_RCODE_NOTZONE;
 		return KNOT_EOUTOFZONE;
 	}
+	const bool is_apex = in_bailiwick == 0;
 
 	if (is_dnssec_protected(rrset->type, is_apex)) {
 		*rcode = KNOT_RCODE_REFUSED;
@@ -658,8 +654,7 @@ static int check_update(const knot_rrset_t *rrset, const knot_pkt_t *query,
 			return KNOT_EMALF;
 		}
 	} else if (rrset->rclass == KNOT_CLASS_NONE) {
-		if ((knot_rdata_ttl(knot_rdataset_at(&rrset->rrs, 0)) != 0) ||
-		    knot_rrtype_is_metatype(rrset->type)) {
+		if (rrset->ttl != 0 || knot_rrtype_is_metatype(rrset->type)) {
 			*rcode = KNOT_RCODE_FORMERR;
 			return KNOT_EMALF;
 		}
@@ -696,7 +691,7 @@ static uint16_t ret_to_rcode(int ret)
 {
 	if (ret == KNOT_EMALF) {
 		return KNOT_RCODE_FORMERR;
-	} else if (ret == KNOT_EDENIED || ret == KNOT_ETTL) {
+	} else if (ret == KNOT_EDENIED) {
 		return KNOT_RCODE_REFUSED;
 	} else {
 		return KNOT_RCODE_SERVFAIL;
@@ -744,7 +739,7 @@ int ddns_process_update(const zone_t *zone, const knot_pkt_t *query,
 		return KNOT_EINVAL;
 	}
 
-	uint32_t sn_old = knot_soa_serial(zone_update_from(update));
+	uint32_t sn_old = knot_soa_serial(zone_update_from(update)->rdata);
 
 	// Process all RRs in the authority section.
 	const knot_pktsection_t *authority = knot_pkt_section(query, KNOT_AUTHORITY);
