@@ -198,9 +198,6 @@ static int adjust_pointers(zone_node_t **tnode, void *data)
 		args->first_node = node;
 	}
 
-	// clear Removed NSEC flag so that no relicts remain
-	node->flags &= ~NODE_FLAGS_REMOVED_NSEC;
-
 	// check if this node is not a wildcard child of its parent
 	if (knot_dname_is_wildcard(node->owner)) {
 		assert(node->parent != NULL);
