@@ -1,4 +1,4 @@
-/*  Copyright (C) 2013 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2015 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,18 +17,12 @@
 #pragma once
 
 #include "knot/conf/conf.h"
-#include "knot/zone/zonedb.h"
-
-struct server;
+#include "knot/server/server.h"
 
 /*!
  * \brief Update zone database according to configuration.
  *
  * \param[in] conf Configuration.
  * \param[in] server Server instance.
- *
- * \retval KNOT_EOK
- * \retval KNOT_EINVAL
- * \retval KNOT_ERROR
  */
-int zonedb_reload(conf_t *conf, struct server *server);
+void zonedb_reload(conf_t *conf, server_t *server);
