@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,12 +17,12 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <limits.h>
 #include <lmdb.h>
 
-#include "dnssec/lib/dnssec/error.h"
 #include "libknot/attribute.h"
 #include "libknot/error.h"
-#include "dnssec/error.h"
+#include "libdnssec/error.h"
 
 struct error {
 	int code;
@@ -49,47 +49,35 @@ static const struct error errors[] = {
 
 	/* General errors. */
 	{ KNOT_ERROR,        "failed" },
-	{ KNOT_ENOTRUNNING,  "resource is not running" },
 	{ KNOT_EPARSEFAIL,   "parser failed" },
 	{ KNOT_ESEMCHECK,    "semantic check" },
-	{ KNOT_EEXPIRED,     "resource is expired" },
 	{ KNOT_EUPTODATE,    "zone is up-to-date" },
 	{ KNOT_EFEWDATA,     "not enough data to parse" },
 	{ KNOT_ESPACE,       "not enough space provided" },
 	{ KNOT_EMALF,        "malformed data" },
-	{ KNOT_ECRYPTO,      "error in crypto library" },
 	{ KNOT_ENSEC3PAR,    "missing or wrong NSEC3PARAM record" },
 	{ KNOT_ENSEC3CHAIN,  "missing or wrong NSEC3 chain in the zone" },
 	{ KNOT_EOUTOFZONE,   "name does not belong to the zone" },
-	{ KNOT_EHASH,        "error in hash table" },
 	{ KNOT_EZONEINVAL,   "invalid zone file" },
-	{ KNOT_EZONENOENT,   "zone file not found" },
 	{ KNOT_ENOZONE,      "no such zone found" },
 	{ KNOT_ENONODE,      "no such node in zone found" },
 	{ KNOT_ENORECORD,    "no such record in zone found" },
 	{ KNOT_ENOMASTER,    "no usable master" },
-	{ KNOT_EDNAMEPTR,    "domain name pointer larger than allowed" },
-	{ KNOT_EPAYLOAD,     "invalid EDNS payload size" },
 	{ KNOT_EPREREQ,      "UPDATE prerequisity not met" },
 	{ KNOT_ETTL,         "TTL mismatch" },
 	{ KNOT_ENOXFR,       "transfer was not sent" },
-	{ KNOT_ENOIXFR,      "transfer is not IXFR (is in AXFR format)" },
-	{ KNOT_EXFRREFUSED,  "zone transfer refused by the server" },
 	{ KNOT_EDENIED,      "not allowed" },
 	{ KNOT_ECONN,        "connection reset" },
 	{ KNOT_ETIMEOUT,     "connection timeout" },
-	{ KNOT_EIXFRSPACE,   "IXFR reply did not fit in" },
-	{ KNOT_ECNAME,       "CNAME loop found in zone" },
 	{ KNOT_ENODIFF,      "cannot create zone diff" },
-	{ KNOT_EDSDIGESTLEN, "DS digest length does not match digest type" },
 	{ KNOT_ENOTSIG,      "expected a TSIG or SIG(0)" },
 	{ KNOT_ELIMIT,       "exceeded response rate limit" },
 	{ KNOT_EZONESIZE,    "zone size exceeded" },
-	{ KNOT_EWRITABLE,    "file is not writable" },
 	{ KNOT_EOF,          "end of file" },
 	{ KNOT_ESYSTEM,      "system error" },
 	{ KNOT_EFILE,        "file error" },
 	{ KNOT_ESOAINVAL,    "SOA mismatch" },
+	{ KNOT_ETRAIL,       "trailing data" },
 
 	/* Control states. */
 	{ KNOT_CTL_ESTOP,     "stopping server" },
@@ -114,11 +102,6 @@ static const struct error errors[] = {
 	{ KNOT_TSIG_EBADTIME,  "TSIG out of time window" },
 	{ KNOT_TSIG_EBADTRUNC, "TSIG bad truncation" },
 
-	/* Key parsing errors. */
-	{ KNOT_KEY_EPUBLIC_KEY_OPEN,    "cannot open public key file" },
-	{ KNOT_KEY_EPRIVATE_KEY_OPEN,   "cannot open private key file" },
-	{ KNOT_KEY_EPUBLIC_KEY_INVALID, "public key file is invalid" },
-
 	/* DNSSEC errors. */
 	{ KNOT_DNSSEC_ENOKEY,          "no keys for signing" },
 	{ KNOT_DNSSEC_EMISSINGKEYTYPE, "missing active KSK or ZSK" },
@@ -181,8 +164,6 @@ int knot_error_from_libdnssec(int libdnssec_errcode)
 		return KNOT_ERROR;
 	case DNSSEC_MALFORMED_DATA:
 		return KNOT_EMALF;
-	case DNSSEC_OUT_OF_RANGE:
-		return KNOT_ERANGE;
 	case DNSSEC_NOT_FOUND:
 		return KNOT_ENOENT;
 	case DNSSEC_NO_PUBLIC_KEY:
@@ -203,6 +184,9 @@ const char *knot_strerror(int code)
 	const char *msg;
 
 	switch (code) {
+	case INT_MIN: // Cannot convert to a positive value.
+		code = KNOT_ERROR;
+		// FALLTHROUGH
 	case KNOT_ERROR_MIN ... KNOT_EOK:
 		msg = lookup_message(code); break;
 	case DNSSEC_ERROR_MIN ... DNSSEC_ERROR_MAX:
@@ -216,6 +200,7 @@ const char *knot_strerror(int code)
 	if (msg != NULL) {
 		return msg;
 	} else {
+		// strerror_r would be better but it requires thread local storage.
 		return strerror(abs(code));
 	}
 }
