@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,8 +27,8 @@
 #include "libknot/tsig-op.h"
 #include "libknot/packet/wire.h"
 #include "libknot/packet/rrset-wire.h"
+#include "libknot/wire.h"
 #include "contrib/mempattern.h"
-#include "contrib/wire.h"
 #include "contrib/wire_ctx.h"
 
 /*! \brief Packet RR array growth step. */
@@ -36,16 +36,15 @@
 #define NEXT_RR_COUNT(count) (((count) / NEXT_RR_ALIGN + 1) * NEXT_RR_ALIGN)
 
 /*! \brief Scan packet for RRSet existence. */
-static bool pkt_contains(const knot_pkt_t *packet,
-			 const knot_rrset_t *rrset)
+static bool pkt_contains(const knot_pkt_t *packet, const knot_rrset_t *rrset)
 {
 	assert(packet);
 	assert(rrset);
 
 	for (int i = 0; i < packet->rrset_count; ++i) {
 		const uint16_t type = packet->rr[i].type;
-		const uint8_t *data = packet->rr[i].rrs.data;
-		if (type == rrset->type && data == rrset->rrs.data) {
+		const knot_rdata_t *data = packet->rr[i].rrs.rdata;
+		if (type == rrset->type && data == rrset->rrs.rdata) {
 			return true;
 		}
 	}
@@ -64,24 +63,21 @@ static void pkt_free_data(knot_pkt_t *pkt)
 			knot_rrset_clear(&pkt->rr[i], &pkt->mm);
 		}
 	}
-
-	/* Reset RR count. */
 	pkt->rrset_count = 0;
 
-	/* Reset special types. */
-	pkt->opt_rr = NULL;
-	pkt->tsig_rr = NULL;
-
-	/* Reset TSIG wire reference. */
-	pkt->tsig_wire.pos = NULL;
-	pkt->tsig_wire.len = 0;
+	/* Free EDNS option positions. */
+	mm_free(&pkt->mm, pkt->edns_opts);
+	pkt->edns_opts = 0;
 }
 
 /*! \brief Allocate new wireformat of given length. */
 static int pkt_wire_alloc(knot_pkt_t *pkt, uint16_t len)
 {
 	assert(pkt);
-	assert(len >= KNOT_WIRE_HEADER_SIZE);
+
+	if (len < KNOT_WIRE_HEADER_SIZE) {
+		return KNOT_ERANGE;
+	}
 
 	pkt->wire = mm_alloc(&pkt->mm, len);
 	if (pkt->wire == NULL) {
@@ -90,7 +86,9 @@ static int pkt_wire_alloc(knot_pkt_t *pkt, uint16_t len)
 
 	pkt->flags |= KNOT_PF_FREE;
 	pkt->max_size = len;
+
 	knot_pkt_clear(pkt);
+
 	return KNOT_EOK;
 }
 
@@ -209,34 +207,11 @@ static int pkt_init(knot_pkt_t *pkt, void *wire, uint16_t len, knot_mm_t *mm)
 }
 
 /*! \brief Reset packet parse state. */
-static int pkt_reset_sections(knot_pkt_t *pkt)
+static void sections_reset(knot_pkt_t *pkt)
 {
-	pkt->parsed  = 0;
 	pkt->current = KNOT_ANSWER;
 	memset(pkt->sections, 0, sizeof(pkt->sections));
-	return knot_pkt_begin(pkt, KNOT_ANSWER);
-}
-
-_public_
-void knot_pkt_clear_payload(knot_pkt_t *pkt)
-{
-	if (!pkt) {
-		return;
-	}
-
-	/* Keep question. */
-	pkt->parsed = 0;
-	pkt->reserved = 0;
-	pkt->size = KNOT_WIRE_HEADER_SIZE + knot_pkt_question_size(pkt);
-	knot_wire_set_ancount(pkt->wire, 0);
-	knot_wire_set_nscount(pkt->wire, 0);
-	knot_wire_set_arcount(pkt->wire, 0);
-
-	/* Free RRSets if applicable. */
-	pkt_free_data(pkt);
-
-	/* Reset sections. */
-	pkt_reset_sections(pkt);
+	(void)knot_pkt_begin(pkt, KNOT_ANSWER);
 }
 
 /*! \brief Allocate new packet using memory context. */
@@ -323,6 +298,29 @@ int knot_pkt_copy(knot_pkt_t *dst, const knot_pkt_t *src)
 	return knot_pkt_parse(dst, 0);
 }
 
+static void payload_clear(knot_pkt_t *pkt)
+{
+	assert(pkt);
+
+	/* Keep question. */
+	pkt->parsed = 0;
+	pkt->reserved = 0;
+
+	/* Free RRSets if applicable. */
+	pkt_free_data(pkt);
+
+	/* Reset sections. */
+	sections_reset(pkt);
+
+	/* Reset special types. */
+	pkt->opt_rr = NULL;
+	pkt->tsig_rr = NULL;
+
+	/* Reset TSIG wire reference. */
+	pkt->tsig_wire.pos = NULL;
+	pkt->tsig_wire.len = 0;
+}
+
 _public_
 int knot_pkt_init_response(knot_pkt_t *pkt, const knot_pkt_t *query)
 {
@@ -345,7 +343,11 @@ int knot_pkt_init_response(knot_pkt_t *pkt, const knot_pkt_t *query)
 		knot_wire_set_qdcount(pkt->wire, 0);
 	}
 
-	/* Update size and flags. */
+	/* Update flags and section counters. */
+	knot_wire_set_ancount(pkt->wire, 0);
+	knot_wire_set_nscount(pkt->wire, 0);
+	knot_wire_set_arcount(pkt->wire, 0);
+
 	knot_wire_set_qr(pkt->wire);
 	knot_wire_clear_tc(pkt->wire);
 	knot_wire_clear_ad(pkt->wire);
@@ -354,7 +356,7 @@ int knot_pkt_init_response(knot_pkt_t *pkt, const knot_pkt_t *query)
 	knot_wire_clear_z(pkt->wire);
 
 	/* Clear payload. */
-	knot_pkt_clear_payload(pkt);
+	payload_clear(pkt);
 
 	return KNOT_EOK;
 }
@@ -366,38 +368,37 @@ void knot_pkt_clear(knot_pkt_t *pkt)
 		return;
 	}
 
-	/* Clear payload. */
-	knot_pkt_clear_payload(pkt);
-
 	/* Reset to header size. */
 	pkt->size = KNOT_WIRE_HEADER_SIZE;
 	memset(pkt->wire, 0, pkt->size);
 
+	/* Clear payload. */
+	payload_clear(pkt);
+
 	/* Clear compression context. */
 	compr_clear(&pkt->compr);
 }
 
 _public_
-void knot_pkt_free(knot_pkt_t **pkt)
+void knot_pkt_free(knot_pkt_t *pkt)
 {
-	if (pkt == NULL || *pkt == NULL) {
+	if (pkt == NULL) {
 		return;
 	}
 
 	/* Free temporary RRSets. */
-	pkt_free_data(*pkt);
+	pkt_free_data(pkt);
 
 	/* Free RR/RR info arrays. */
-	mm_free(&(*pkt)->mm, (*pkt)->rr);
-	mm_free(&(*pkt)->mm, (*pkt)->rr_info);
+	mm_free(&pkt->mm, pkt->rr);
+	mm_free(&pkt->mm, pkt->rr_info);
 
-	// free the space for wireformat
-	if ((*pkt)->flags & KNOT_PF_FREE) {
-		mm_free(&(*pkt)->mm, (*pkt)->wire);
+	/* Free the space for wireformat. */
+	if (pkt->flags & KNOT_PF_FREE) {
+		mm_free(&pkt->mm, pkt->wire);
 	}
 
-	mm_free(&(*pkt)->mm, *pkt);
-	*pkt = NULL;
+	mm_free(&pkt->mm, pkt);
 }
 
 _public_
@@ -431,48 +432,6 @@ int knot_pkt_reclaim(knot_pkt_t *pkt, uint16_t size)
 	}
 }
 
-_public_
-uint16_t knot_pkt_question_size(const knot_pkt_t *pkt)
-{
-	if (pkt == NULL || pkt->qname_size == 0) {
-		return 0;
-	}
-
-	return pkt->qname_size + 2 * sizeof(uint16_t);
-}
-
-_public_
-const knot_dname_t *knot_pkt_qname(const knot_pkt_t *pkt)
-{
-	if (pkt == NULL || pkt->qname_size == 0) {
-		return NULL;
-	}
-
-	return pkt->wire + KNOT_WIRE_HEADER_SIZE;
-}
-
-_public_
-uint16_t knot_pkt_qtype(const knot_pkt_t *pkt)
-{
-	if (pkt == NULL || pkt->qname_size == 0) {
-		return 0;
-	}
-
-	unsigned off = KNOT_WIRE_HEADER_SIZE + pkt->qname_size;
-	return wire_read_u16(pkt->wire + off);
-}
-
-_public_
-uint16_t knot_pkt_qclass(const knot_pkt_t *pkt)
-{
-	if (pkt == NULL || pkt->qname_size == 0) {
-		return 0;
-	}
-
-	unsigned off = KNOT_WIRE_HEADER_SIZE + pkt->qname_size + sizeof(uint16_t);
-	return wire_read_u16(pkt->wire + off);
-}
-
 _public_
 int knot_pkt_begin(knot_pkt_t *pkt, knot_section_t section_id)
 {
@@ -531,8 +490,8 @@ int knot_pkt_put_question(knot_pkt_t *pkt, const knot_dname_t *qname, uint16_t q
 }
 
 _public_
-int knot_pkt_put(knot_pkt_t *pkt, uint16_t compr_hint, const knot_rrset_t *rr,
-                 uint16_t flags)
+int knot_pkt_put_rotate(knot_pkt_t *pkt, uint16_t compr_hint, const knot_rrset_t *rr,
+                        uint16_t rotate, uint16_t flags)
 {
 	if (pkt == NULL || rr == NULL) {
 		return KNOT_EINVAL;
@@ -546,7 +505,7 @@ int knot_pkt_put(knot_pkt_t *pkt, uint16_t compr_hint, const knot_rrset_t *rr,
 
 	/* Check for double insertion. */
 	if ((flags & KNOT_PF_CHECKDUP) && pkt_contains(pkt, rr)) {
-		return KNOT_EOK; /*! \todo return rather a number of added RRs */
+		return KNOT_EOK;
 	}
 
 	knot_rrinfo_t *rrinfo = &pkt->rr_info[pkt->rrset_count];
@@ -556,19 +515,26 @@ int knot_pkt_put(knot_pkt_t *pkt, uint16_t compr_hint, const knot_rrset_t *rr,
 	rrinfo->compress_ptr[0] = compr_hint;
 	memcpy(pkt->rr + pkt->rrset_count, rr, sizeof(knot_rrset_t));
 
-	/* Initialize compression context if it did not happen yet. */
-	pkt->compr.rrinfo = rrinfo;
-	if (pkt->compr.suffix.pos == 0) {
-		pkt->compr.suffix.pos = KNOT_WIRE_HEADER_SIZE;
-		pkt->compr.suffix.labels = knot_dname_labels(pkt->compr.wire + pkt->compr.suffix.pos,
-		                                             pkt->compr.wire);
+	/* Disable compression if no QNAME is available. */
+	knot_compr_t *compr = NULL;
+	if (knot_pkt_qname(pkt) != NULL) {
+		/* Initialize compression context if it did not happen yet. */
+		pkt->compr.rrinfo = rrinfo;
+		if (pkt->compr.suffix.pos == 0) {
+			pkt->compr.suffix.pos = KNOT_WIRE_HEADER_SIZE;
+			pkt->compr.suffix.labels =
+				knot_dname_labels(pkt->compr.wire + pkt->compr.suffix.pos,
+				                  pkt->compr.wire);
+		}
+
+		compr = &pkt->compr;
 	}
 
 	uint8_t *pos = pkt->wire + pkt->size;
 	size_t maxlen = pkt_remaining(pkt);
 
 	/* Write RRSet to wireformat. */
-	ret = knot_rrset_to_wire(rr, pos, maxlen, &pkt->compr);
+	ret = knot_rrset_to_wire_extra(rr, pos, maxlen, rotate, compr, flags);
 	if (ret < 0) {
 		/* Truncate packet if required. */
 		if (ret == KNOT_ESPACE && !(flags & KNOT_PF_NOTRUNC)) {
@@ -578,7 +544,7 @@ int knot_pkt_put(knot_pkt_t *pkt, uint16_t compr_hint, const knot_rrset_t *rr,
 	}
 
 	size_t len = ret;
-	uint16_t rr_added = rr->rrs.rr_count;
+	uint16_t rr_added = rr->rrs.count;
 
 	/* Keep reference to special types. */
 	if (rr->type == KNOT_RRTYPE_OPT) {
@@ -595,55 +561,6 @@ int knot_pkt_put(knot_pkt_t *pkt, uint16_t compr_hint, const knot_rrset_t *rr,
 	return KNOT_EOK;
 }
 
-_public_
-const knot_pktsection_t *knot_pkt_section(const knot_pkt_t *pkt,
-                                          knot_section_t section_id)
-{
-	if (pkt == NULL) {
-		return NULL;
-	}
-
-	return &pkt->sections[section_id];
-}
-
-_public_
-const knot_rrset_t *knot_pkt_rr(const knot_pktsection_t *section, uint16_t i)
-{
-	if (section == NULL) {
-		return NULL;
-	}
-
-	return section->pkt->rr + section->pos + i;
-}
-
-_public_
-uint16_t knot_pkt_rr_offset(const knot_pktsection_t *section, uint16_t i)
-{
-	if (section == NULL || section->pkt == NULL) {
-		return -1;
-	}
-
-	return section->pkt->rr_info[section->pos + i].pos;
-}
-
-_public_
-int knot_pkt_parse(knot_pkt_t *pkt, unsigned flags)
-{
-	if (pkt == NULL) {
-		return KNOT_EINVAL;
-	}
-
-	/* Reset parse state. */
-	pkt_reset_sections(pkt);
-
-	int ret = knot_pkt_parse_question(pkt);
-	if (ret == KNOT_EOK) {
-		ret = knot_pkt_parse_payload(pkt, flags);
-	}
-
-	return ret;
-}
-
 _public_
 int knot_pkt_parse_question(knot_pkt_t *pkt)
 {
@@ -691,28 +608,18 @@ int knot_pkt_parse_question(knot_pkt_t *pkt)
 	return KNOT_EOK;
 }
 
-/* \note Private for check_rr_constraints(). */
-#define CHECK_AR_CONSTRAINTS(pkt, rr, var, check_func) \
-	if ((pkt)->current != KNOT_ADDITIONAL) { \
-		return KNOT_EMALF; \
-	} else if ((pkt)->var != NULL) { \
-		return KNOT_EMALF; \
-	} else if (!check_func(rr)) { \
-		return KNOT_EMALF; \
-	} else { \
-		(pkt)->var = rr; \
-	}
-
 /*! \brief Check constraints (position, uniqueness, validity) for special types
  *         (TSIG, OPT).
  */
 static int check_rr_constraints(knot_pkt_t *pkt, knot_rrset_t *rr, size_t rr_size,
                                 unsigned flags)
 {
-	/* Check RR constraints. */
 	switch (rr->type) {
 	case KNOT_RRTYPE_TSIG:
-		CHECK_AR_CONSTRAINTS(pkt, rr, tsig_rr, knot_tsig_rdata_is_ok);
+		if (pkt->current != KNOT_ADDITIONAL || pkt->tsig_rr != NULL ||
+		    !knot_tsig_rdata_is_ok(rr)) {
+			return KNOT_EMALF;
+		}
 
 		/* Strip TSIG RR from wireformat and decrease ARCOUNT. */
 		if (!(flags & KNOT_PF_KEEPWIRE)) {
@@ -722,9 +629,16 @@ static int check_rr_constraints(knot_pkt_t *pkt, knot_rrset_t *rr, size_t rr_siz
 			pkt->tsig_wire.len = rr_size;
 			knot_wire_set_arcount(pkt->wire, knot_wire_get_arcount(pkt->wire) - 1);
 		}
+
+		pkt->tsig_rr = rr;
 		break;
 	case KNOT_RRTYPE_OPT:
-		CHECK_AR_CONSTRAINTS(pkt, rr, opt_rr, knot_edns_check_record);
+		if (pkt->current != KNOT_ADDITIONAL || pkt->opt_rr != NULL ||
+		    knot_edns_get_options(rr, &pkt->edns_opts, &pkt->mm) != KNOT_EOK) {
+			return KNOT_EMALF;
+		}
+
+		pkt->opt_rr = rr;
 		break;
 	default:
 		break;
@@ -733,14 +647,9 @@ static int check_rr_constraints(knot_pkt_t *pkt, knot_rrset_t *rr, size_t rr_siz
 	return KNOT_EOK;
 }
 
-#undef CHECK_AR_CONSTRAINTS
-
-_public_
-int knot_pkt_parse_rr(knot_pkt_t *pkt, unsigned flags)
+static int parse_rr(knot_pkt_t *pkt, unsigned flags)
 {
-	if (pkt == NULL) {
-		return KNOT_EINVAL;
-	}
+	assert(pkt);
 
 	if (pkt->parsed >= pkt->size) {
 		return KNOT_EFEWDATA;
@@ -761,7 +670,7 @@ int knot_pkt_parse_rr(knot_pkt_t *pkt, unsigned flags)
 	size_t rr_size = pkt->parsed;
 	knot_rrset_t *rr = &pkt->rr[pkt->rrset_count];
 	ret = knot_rrset_rr_from_wire(pkt->wire, &pkt->parsed, pkt->size,
-	                              &pkt->mm, rr, !(flags & KNOT_PF_NOCANON));
+	                              rr, &pkt->mm, !(flags & KNOT_PF_NOCANON));
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
@@ -777,20 +686,16 @@ int knot_pkt_parse_rr(knot_pkt_t *pkt, unsigned flags)
 	return check_rr_constraints(pkt, rr, rr_size, flags);
 }
 
-_public_
-int knot_pkt_parse_section(knot_pkt_t *pkt, unsigned flags)
+static int parse_section(knot_pkt_t *pkt, unsigned flags)
 {
-	if (pkt == NULL) {
-		return KNOT_EINVAL;
-	}
+	assert(pkt);
 
-	int ret = KNOT_EOK;
 	uint16_t rr_parsed = 0;
 	uint16_t rr_count = pkt_rr_wirecount(pkt, pkt->current);
 
 	/* Parse all RRs belonging to the section. */
 	for (rr_parsed = 0; rr_parsed < rr_count; ++rr_parsed) {
-		ret = knot_pkt_parse_rr(pkt, flags);
+		int ret = parse_rr(pkt, flags);
 		if (ret != KNOT_EOK) {
 			return ret;
 		}
@@ -799,14 +704,10 @@ int knot_pkt_parse_section(knot_pkt_t *pkt, unsigned flags)
 	return KNOT_EOK;
 }
 
-_public_
-int knot_pkt_parse_payload(knot_pkt_t *pkt, unsigned flags)
+static int parse_payload(knot_pkt_t *pkt, unsigned flags)
 {
-	if (pkt == NULL) {
-		return KNOT_EINVAL;
-	}
-
-	assert(pkt->wire != NULL);
+	assert(pkt);
+	assert(pkt->wire);
 	assert(pkt->size > 0);
 
 	/* Reserve memory in advance to avoid resizing. */
@@ -828,7 +729,7 @@ int knot_pkt_parse_payload(knot_pkt_t *pkt, unsigned flags)
 		if (ret != KNOT_EOK) {
 			return ret;
 		}
-		ret = knot_pkt_parse_section(pkt, flags);
+		ret = parse_section(pkt, flags);
 		if (ret != KNOT_EOK) {
 			return ret;
 		}
@@ -838,19 +739,37 @@ int knot_pkt_parse_payload(knot_pkt_t *pkt, unsigned flags)
 	const knot_pktsection_t *ar = knot_pkt_section(pkt, KNOT_ADDITIONAL);
 	if (pkt->tsig_rr != NULL) {
 		const knot_rrset_t *last_rr = knot_pkt_rr(ar, ar->count - 1);
-		if (ar->count > 0 && pkt->tsig_rr->rrs.data != last_rr->rrs.data) {
+		if (ar->count > 0 && pkt->tsig_rr->rrs.rdata != last_rr->rrs.rdata) {
 			return KNOT_EMALF;
 		}
 	}
 
 	/* Check for trailing garbage. */
 	if (pkt->parsed < pkt->size) {
-		return KNOT_EMALF;
+		return KNOT_ETRAIL;
 	}
 
 	return KNOT_EOK;
 }
 
+_public_
+int knot_pkt_parse(knot_pkt_t *pkt, unsigned flags)
+{
+	if (pkt == NULL) {
+		return KNOT_EINVAL;
+	}
+
+	/* Reset parse state. */
+	sections_reset(pkt);
+
+	int ret = knot_pkt_parse_question(pkt);
+	if (ret == KNOT_EOK) {
+		ret = parse_payload(pkt, flags);
+	}
+
+	return ret;
+}
+
 _public_
 uint16_t knot_pkt_ext_rcode(const knot_pkt_t *pkt)
 {
