@@ -1,4 +1,4 @@
-/*  Copyright (C) 2016 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *
  * \brief TSIG manipulation.
  *
- * \addtogroup libknot
+ * \addtogroup rrtype
  * @{
  */
 
@@ -26,9 +26,8 @@
 
 #include <stdint.h>
 
-#include "dnssec/binary.h"
-#include "dnssec/tsig.h"
-#include "libknot/binary.h"
+#include "libdnssec/binary.h"
+#include "libdnssec/tsig.h"
 #include "libknot/consts.h"
 #include "libknot/rrset.h"
 #include "libknot/tsig.h"
@@ -96,9 +95,20 @@ size_t knot_tsig_rdata_tsig_variables_length(const knot_rrset_t *tsig);
 
 size_t knot_tsig_rdata_tsig_timers_length(void);
 
+/*!
+ * \brief Return standard TSIG RRSET wire size for given algorithm.
+ *
+ * \param key Signing key descriptor.
+ *
+ * \return RRSET wire size.
+ */
+size_t knot_tsig_wire_size(const knot_tsig_key_t *key);
+
 /*!
  * \brief Return TSIG RRSET maximum wire size for given algorithm.
  *
+ * This size is reached if error reply with BADTIME.
+ *
  * \param key Signing key descriptor.
  *
  * \return RRSET wire size.
@@ -106,6 +116,6 @@ size_t knot_tsig_rdata_tsig_timers_length(void);
 size_t knot_tsig_wire_maxsize(const knot_tsig_key_t *key);
 
 /*! \todo Documentation. */
-int knot_tsig_rdata_is_ok(const knot_rrset_t *tsig);
+bool knot_tsig_rdata_is_ok(const knot_rrset_t *tsig);
 
 /*! @} */
