@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,8 +19,8 @@
 #include <time.h>
 #include <stdint.h>
 
-#include "dnssec/error.h"
-#include "dnssec/tsig.h"
+#include "libdnssec/error.h"
+#include "libdnssec/tsig.h"
 #include "libknot/attribute.h"
 #include "libknot/tsig-op.h"
 #include "libknot/errcode.h"
@@ -29,8 +29,8 @@
 #include "libknot/packet/wire.h"
 #include "libknot/consts.h"
 #include "libknot/packet/rrset-wire.h"
+#include "libknot/wire.h"
 #include "contrib/string.h"
-#include "contrib/wire.h"
 
 const int KNOT_TSIG_MAX_DIGEST_SIZE = 64;    // size of HMAC-SHA512 digest
 const uint16_t KNOT_TSIG_FUDGE_DEFAULT = 300;  // default Fudge value
@@ -65,7 +65,7 @@ static int check_key(const knot_rrset_t *tsig_rr, const knot_tsig_key_t *tsig_ke
 		return KNOT_EMALF;
 	}
 
-	if (knot_dname_cmp(tsig_name, tsig_key->name) != 0) {
+	if (!knot_dname_is_equal(tsig_name, tsig_key->name)) {
 		return KNOT_TSIG_EBADKEY;
 	}
 
@@ -154,11 +154,11 @@ static int write_tsig_variables(uint8_t *wire, const knot_rrset_t *tsig_rr)
 	/*!< \todo which order? */
 
 	/* Copy class. */
-	wire_write_u16(wire + offset, tsig_rr->rclass);
+	knot_wire_write_u16(wire + offset, tsig_rr->rclass);
 	offset += sizeof(uint16_t);
 
 	/* Copy TTL - always 0. */
-	wire_write_u32(wire + offset, knot_rdata_ttl(knot_rdataset_at(&tsig_rr->rrs, 0)));
+	knot_wire_write_u32(wire + offset, tsig_rr->ttl);
 	offset += sizeof(uint32_t);
 
 	/* Copy alg name. */
@@ -170,19 +170,17 @@ static int write_tsig_variables(uint8_t *wire, const knot_rrset_t *tsig_rr)
 	/* Te algorithm name must be in canonical form, i.e. in lowercase. */
 	uint8_t *alg_name_wire = wire + offset;
 	offset += knot_dname_to_wire(alg_name_wire, alg_name, KNOT_DNAME_MAXLEN);
-	if (knot_dname_to_lower(alg_name_wire) != KNOT_EOK) {
-		return KNOT_EINVAL;
-	}
+	knot_dname_to_lower(alg_name_wire);
 
 	/* Following data are written in network order. */
 	/* Time signed. */
-	wire_write_u48(wire + offset, knot_tsig_rdata_time_signed(tsig_rr));
+	knot_wire_write_u48(wire + offset, knot_tsig_rdata_time_signed(tsig_rr));
 	offset += 6;
 	/* Fudge. */
-	wire_write_u16(wire + offset, knot_tsig_rdata_fudge(tsig_rr));
+	knot_wire_write_u16(wire + offset, knot_tsig_rdata_fudge(tsig_rr));
 	offset += sizeof(uint16_t);
 	/* TSIG error. */
-	wire_write_u16(wire + offset, knot_tsig_rdata_error(tsig_rr));
+	knot_wire_write_u16(wire + offset, knot_tsig_rdata_error(tsig_rr));
 	offset += sizeof(uint16_t);
 	/* Get other data length. */
 	uint16_t other_data_length = knot_tsig_rdata_other_data_length(tsig_rr);
@@ -196,7 +194,7 @@ static int write_tsig_variables(uint8_t *wire, const knot_rrset_t *tsig_rr)
 	 * We cannot write the whole other_data, as it contains its length in
 	 * machine order.
 	 */
-	wire_write_u16(wire + offset, other_data_length);
+	knot_wire_write_u16(wire + offset, other_data_length);
 	offset += sizeof(uint16_t);
 
 	/* Skip the length. */
@@ -212,9 +210,9 @@ static int wire_write_timers(uint8_t *wire, const knot_rrset_t *tsig_rr)
 	}
 
 	//write time signed
-	wire_write_u48(wire, knot_tsig_rdata_time_signed(tsig_rr));
+	knot_wire_write_u48(wire, knot_tsig_rdata_time_signed(tsig_rr));
 	//write fudge
-	wire_write_u16(wire + 6, knot_tsig_rdata_fudge(tsig_rr));
+	knot_wire_write_u16(wire + 6, knot_tsig_rdata_fudge(tsig_rr));
 
 	return KNOT_EOK;
 }
@@ -249,7 +247,7 @@ static int create_sign_wire(const uint8_t *msg, size_t msg_len,
 
 	/* Copy the request MAC - should work even if NULL. */
 	if (request_mac_len > 0) {
-		wire_write_u16(pos, request_mac_len);
+		knot_wire_write_u16(pos, request_mac_len);
 		pos += 2;
 		memcpy(pos, request_mac, request_mac_len);
 	}
@@ -303,7 +301,7 @@ static int create_sign_wire_next(const uint8_t *msg, size_t msg_len,
 	memset(wire, 0, wire_len);
 
 	/* Copy the request MAC - should work even if NULL. */
-	wire_write_u16(wire, prev_mac_len);
+	knot_wire_write_u16(wire, prev_mac_len);
 	memcpy(wire + 2, prev_mac, prev_mac_len);
 	/* Copy the original message. */
 	memcpy(wire + prev_mac_len + 2, msg, msg_len);
@@ -340,7 +338,7 @@ int knot_tsig_sign(uint8_t *msg, size_t *msg_len, size_t msg_max_len,
 	}
 
 	knot_rrset_t *tmp_tsig = knot_rrset_new(key->name, KNOT_RRTYPE_TSIG,
-	                                        KNOT_CLASS_ANY, NULL);
+	                                        KNOT_CLASS_ANY, 0, NULL);
 	if (!tmp_tsig) {
 		return KNOT_ENOMEM;
 	}
@@ -363,7 +361,7 @@ int knot_tsig_sign(uint8_t *msg, size_t *msg_len, size_t msg_max_len,
 		/* Store current time into Other data. */
 		uint8_t time_signed[6];
 		time_t now = time(NULL);
-		wire_write_u48(time_signed, now);
+		knot_wire_write_u48(time_signed, now);
 
 		knot_tsig_rdata_set_other_data(tmp_tsig, 6, time_signed);
 	} else {
@@ -385,7 +383,7 @@ int knot_tsig_sign(uint8_t *msg, size_t *msg_len, size_t msg_max_len,
 	                           request_mac, request_mac_len,
 	                           digest_tmp, &digest_tmp_len, tmp_tsig, key);
 	if (ret != KNOT_EOK) {
-		knot_rrset_free(&tmp_tsig, NULL);
+		knot_rrset_free(tmp_tsig, NULL);
 		return ret;
 	}
 
@@ -397,13 +395,13 @@ int knot_tsig_sign(uint8_t *msg, size_t *msg_len, size_t msg_max_len,
 	                         msg_max_len - *msg_len, NULL);
 	if (ret < 0) {
 		*digest_len = 0;
-		knot_rrset_free(&tmp_tsig, NULL);
+		knot_rrset_free(tmp_tsig, NULL);
 		return ret;
 	}
 
 	size_t tsig_wire_len = ret;
 
-	knot_rrset_free(&tmp_tsig, NULL);
+	knot_rrset_free(tmp_tsig, NULL);
 
 	*msg_len += tsig_wire_len;
 
@@ -431,7 +429,7 @@ int knot_tsig_sign_next(uint8_t *msg, size_t *msg_len, size_t msg_max_len,
 	uint8_t digest_tmp[KNOT_TSIG_MAX_DIGEST_SIZE];
 	size_t digest_tmp_len = 0;
 	knot_rrset_t *tmp_tsig = knot_rrset_new(key->name, KNOT_RRTYPE_TSIG,
-	                                        KNOT_CLASS_ANY, NULL);
+	                                        KNOT_CLASS_ANY, 0, NULL);
 	if (!tmp_tsig) {
 		return KNOT_ENOMEM;
 	}
@@ -447,13 +445,13 @@ int knot_tsig_sign_next(uint8_t *msg, size_t *msg_len, size_t msg_max_len,
 	size_t wire_len = prev_digest_len + to_sign_len + KNOT_TSIG_TIMERS_LENGTH + 2;
 	uint8_t *wire = malloc(wire_len);
 	if (!wire) {
-		knot_rrset_free(&tmp_tsig, NULL);
+		knot_rrset_free(tmp_tsig, NULL);
 		return KNOT_ENOMEM;
 	}
 	memset(wire, 0, wire_len);
 
 	/* Write previous digest length. */
-	wire_write_u16(wire, prev_digest_len);
+	knot_wire_write_u16(wire, prev_digest_len);
 	/* Write previous digest. */
 	memcpy(wire + 2, prev_digest, prev_digest_len);
 	/* Write original message. */
@@ -464,13 +462,13 @@ int knot_tsig_sign_next(uint8_t *msg, size_t *msg_len, size_t msg_max_len,
 	int ret = compute_digest(wire, wire_len, digest_tmp, &digest_tmp_len, key);
 	free(wire);
 	if (ret != KNOT_EOK) {
-		knot_rrset_free(&tmp_tsig, NULL);
+		knot_rrset_free(tmp_tsig, NULL);
 		*digest_len = 0;
 		return ret;
 	}
 
 	if (digest_tmp_len > *digest_len) {
-		knot_rrset_free(&tmp_tsig, NULL);
+		knot_rrset_free(tmp_tsig, NULL);
 		*digest_len = 0;
 		return KNOT_ESPACE;
 	}
@@ -487,14 +485,14 @@ int knot_tsig_sign_next(uint8_t *msg, size_t *msg_len, size_t msg_max_len,
 	ret = knot_rrset_to_wire(tmp_tsig, msg + *msg_len,
 	                         msg_max_len - *msg_len, NULL);
 	if (ret < 0) {
-		knot_rrset_free(&tmp_tsig, NULL);
+		knot_rrset_free(tmp_tsig, NULL);
 		*digest_len = 0;
 		return ret;
 	}
 
 	size_t tsig_wire_size = ret;
 
-	knot_rrset_free(&tmp_tsig, NULL);
+	knot_rrset_free(tmp_tsig, NULL);
 
 	*msg_len += tsig_wire_size;
 	uint16_t arcount = knot_wire_get_arcount(msg);
@@ -545,7 +543,7 @@ static int check_digest(const knot_rrset_t *tsig_rr,
 
 	uint8_t digest_tmp[KNOT_TSIG_MAX_DIGEST_SIZE];
 	size_t digest_tmp_len = 0;
-	assert(tsig_rr->rrs.rr_count > 0);
+	assert(tsig_rr->rrs.count > 0);
 
 	if (use_times) {
 		/* Wire is not a single packet, TSIG RRs must be stripped already. */
@@ -560,7 +558,7 @@ static int check_digest(const knot_rrset_t *tsig_rr,
 		                       tsig_rr, tsig_key);
 	}
 
-	assert(tsig_rr->rrs.rr_count > 0);
+	assert(tsig_rr->rrs.count > 0);
 	free(wire_to_sign);
 
 	if (ret != KNOT_EOK) {
@@ -637,7 +635,7 @@ int knot_tsig_add(uint8_t *msg, size_t *msg_len, size_t msg_max_len,
 
 	/*! \todo What key to use, when we do not sign? Does this even work? */
 	knot_rrset_t *tmp_tsig = knot_rrset_new(tsig_rr->owner, KNOT_RRTYPE_TSIG,
-		                                KNOT_CLASS_ANY, NULL);
+		                                KNOT_CLASS_ANY, 0, NULL);
 	if (!tmp_tsig) {
 		return KNOT_ENOMEM;
 	}
@@ -661,7 +659,7 @@ int knot_tsig_add(uint8_t *msg, size_t *msg_len, size_t msg_max_len,
 	int ret = knot_tsig_append(msg, msg_len, msg_max_len, tmp_tsig);
 
 	/* key_name already referenced in RRSet, no need to free separately. */
-	knot_rrset_free(&tmp_tsig, NULL);
+	knot_rrset_free(tmp_tsig, NULL);
 
 	return ret;
 }
