@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,16 +18,15 @@
 #include <inttypes.h>
 #include <limits.h>
 #include <stdlib.h>
+#include <stdio.h>
 
 #include "libknot/yparser/yptrafo.h"
 #include "libknot/attribute.h"
 #include "libknot/consts.h"
 #include "libknot/dname.h"
-#include "libknot/lookup.h"
 #include "contrib/base64.h"
 #include "contrib/ctype.h"
 #include "contrib/sockaddr.h"
-#include "contrib/wire.h"
 #include "contrib/wire_ctx.h"
 
 enum {
@@ -639,7 +638,8 @@ int yp_option_to_bin(
 	YP_CHECK_PARAMS_BIN;
 
 	while (opts->name != NULL) {
-		if (strncasecmp((char *)in->position, opts->name, YP_LEN) == 0) {
+		if (YP_LEN == strlen(opts->name) &&
+		    strncasecmp((char *)in->position, opts->name, YP_LEN) == 0) {
 			wire_ctx_write_u8(out, opts->id);
 			wire_ctx_skip(in, YP_LEN);
 			YP_CHECK_RET;
@@ -703,9 +703,7 @@ int yp_dname_to_bin(
 	}
 
 	// Convert the result to lower case.
-	if (knot_dname_to_lower(out->position) != KNOT_EOK) {
-		return KNOT_EINVAL;
-	}
+	knot_dname_to_lower(out->position);
 
 	wire_ctx_skip(out, ret);
 
@@ -1060,13 +1058,6 @@ struct sockaddr_storage yp_addr_noport(
 	return ss;
 }
 
-_public_
-int64_t yp_int(
-	const uint8_t *data)
-{
-	return (int64_t)wire_read_u64(data);
-}
-
 _public_
 struct sockaddr_storage yp_addr(
 	const uint8_t *data,
@@ -1090,7 +1081,7 @@ struct sockaddr_storage yp_addr(
 	}
 
 	if (addr_len > 0) {
-		int64_t port = wire_read_u64(data + sizeof(uint8_t) + addr_len);
+		int64_t port = knot_wire_read_u64(data + sizeof(uint8_t) + addr_len);
 		if (port >= 0) {
 			sockaddr_port_set((struct sockaddr *)&ss, port);
 			*no_port = false;
@@ -1101,10 +1092,3 @@ struct sockaddr_storage yp_addr(
 
 	return ss;
 }
-
-_public_
-const size_t yp_bin_len(
-	const uint8_t *data)
-{
-	return wire_read_u16(data);
-}
