@@ -1,4 +1,4 @@
-/*  Copyright (C) 2016 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -41,16 +41,23 @@ typedef struct {
 	char	*service;
 } srv_info_t;
 
+typedef enum {
+	NET_FLAGS_NONE = 0,
+	NET_FLAGS_FASTOPEN = 1 << 0,
+} net_flags_t;
+
 typedef struct {
 	/*! Socket descriptor. */
-	int	sockfd;
+	int sockfd;
 
 	/*! IP protocol type. */
-	int	iptype;
+	int iptype;
 	/*! Socket type. */
-	int	socktype;
+	int socktype;
 	/*! Timeout for all network operations. */
-	int	wait;
+	int wait;
+	/*! Connection flags. */
+	net_flags_t flags;
 
 	/*! Local interface parameters. */
 	const srv_info_t *local;
@@ -130,7 +137,7 @@ const char *get_sockname(const int socktype);
  *
  * \param ss		Socket address storage.
  * \param socktype	Socket type (SOCK_STREAM or SOCK_DGRAM).
- * \param dst[out]	Output string.
+ * \param dst		Output string.
  */
 void get_addr_str(const struct sockaddr_storage *ss,
                   const int                     socktype,
@@ -145,6 +152,7 @@ void get_addr_str(const struct sockaddr_storage *ss,
  * \param socktype	Socket type.
  * \param wait		Network timeout interval.
  * \param tls_params	TLS parameters.
+ * \param flags		Connection flags.
  * \param net		Network structure to initialize.
  *
  * \retval KNOT_EOK	if success.
@@ -155,6 +163,7 @@ int net_init(const srv_info_t    *local,
              const int           iptype,
              const int           socktype,
              const int           wait,
+             const net_flags_t   flags,
              const tls_params_t  *tls_params,
              net_t               *net);
 
