@@ -136,12 +136,12 @@ static void fill_remote_addr(net_t *net, Dnstap__Message *message, bool is_initi
 	get_addr_str(&ss, sock_type, &net->remote_str);
 }
 
-static void process_dnstap(const query_t *query)
+static int process_dnstap(const query_t *query)
 {
 	dt_reader_t *reader = query->dt_reader;
 
 	if (query->dt_reader == NULL) {
-		return;
+		return -1;
 	}
 
 	bool first_message = true;
@@ -238,6 +238,8 @@ static void process_dnstap(const query_t *query)
 		knot_pkt_free(pkt);
 		dt_reader_free_frame(reader, &frame);
 	}
+
+	return 0;
 }
 #endif // USE_DNSTAP
 
@@ -665,7 +667,7 @@ static int process_query_packet(const knot_pkt_t      *query,
 		if (reply == NULL) {
 			ERR("internal error (%s)\n", knot_strerror(KNOT_ENOMEM));
 			net_close(net);
-			return 0;
+			return -1;
 		}
 
 		// Parse reply to the packet structure.
@@ -673,7 +675,7 @@ static int process_query_packet(const knot_pkt_t      *query,
 			ERR("malformed reply packet from %s\n", net->remote_str);
 			knot_pkt_free(reply);
 			net_close(net);
-			return 0;
+			return -1;
 		}
 
 		// Compare reply header id.
@@ -768,23 +770,18 @@ static int process_query_packet(const knot_pkt_t      *query,
 	return 0;
 }
 
-static void process_query(const query_t *query)
+static int process_query(const query_t *query)
 {
 	node_t     *server = NULL;
 	knot_pkt_t *out_packet;
 	net_t      net;
 	int        ret;
 
-	if (query == NULL) {
-		DBG_NULL;
-		return;
-	}
-
 	// Create query packet.
 	out_packet = create_query_packet(query);
 	if (out_packet == NULL) {
 		ERR("can't create query packet\n");
-		return;
+		return -1;
 	}
 
 	// Sign the query.
@@ -792,7 +789,7 @@ static void process_query(const query_t *query)
 	ret = sign_query(out_packet, query, &sign_ctx);
 	if (ret != KNOT_EOK) {
 		ERR("can't sign the packet (%s)\n", knot_strerror(ret));
-		return;
+		return -1;
 	}
 
 	// Get connection parameters.
@@ -828,7 +825,7 @@ static void process_query(const query_t *query)
 				// If error try next resolved address.
 				if (ret != 0) {
 					net.srv = (net.srv)->ai_next;
-					if (net.srv != NULL) {
+					if (net.srv != NULL && query->style.show_query) {
 						printf("\n");
 					}
 
@@ -843,30 +840,35 @@ static void process_query(const query_t *query)
 				net_clean(&net);
 				sign_context_deinit(&sign_ctx);
 				knot_pkt_free(out_packet);
-				return;
+				return 0;
 			}
 
 			if (i < query->retries) {
-				printf("\n");
 				DBG("retrying server %s@%s(%s)\n",
 				    remote->name, remote->service,
 				    get_sockname(socktype));
+
+				if (query->style.show_query) {
+					printf("\n");
+				}
 			}
 
 			net_clean(&net);
 		}
 
-		WARN("failed to query server %s@%s(%s)\n",
-		     remote->name, remote->service, get_sockname(socktype));
+		ERR("failed to query server %s@%s(%s)\n",
+		    remote->name, remote->service, get_sockname(socktype));
 
 		// If not last server, print separation.
-		if (server->next->next) {
+		if (server->next->next && query->style.show_query) {
 			printf("\n");
 		}
 	}
 
 	sign_context_deinit(&sign_ctx);
 	knot_pkt_free(out_packet);
+
+	return -1;
 }
 
 static int process_xfr_packet(const knot_pkt_t      *query,
@@ -959,7 +961,7 @@ static int process_xfr_packet(const knot_pkt_t      *query,
 		if (reply == NULL) {
 			ERR("internal error (%s)\n", knot_strerror(KNOT_ENOMEM));
 			net_close(net);
-			return 0;
+			return -1;
 		}
 
 		// Parse reply to the packet structure.
@@ -967,7 +969,7 @@ static int process_xfr_packet(const knot_pkt_t      *query,
 			ERR("malformed reply packet from %s\n", net->remote_str);
 			knot_pkt_free(reply);
 			net_close(net);
-			return 0;
+			return -1;
 		}
 
 		// Compare reply header id.
@@ -975,7 +977,7 @@ static int process_xfr_packet(const knot_pkt_t      *query,
 			ERR("reply ID mismatch from %s\n", net->remote_str);
 			knot_pkt_free(reply);
 			net_close(net);
-			return 0;
+			return -1;
 		}
 
 		// Print leading transfer information.
@@ -989,7 +991,7 @@ static int process_xfr_packet(const knot_pkt_t      *query,
 			    knot_pkt_ext_rcode_name(reply));
 			knot_pkt_free(reply);
 			net_close(net);
-			return 0;
+			return -1;
 		}
 
 		// The first message has a special treatment.
@@ -1009,7 +1011,7 @@ static int process_xfr_packet(const knot_pkt_t      *query,
 					    net->remote_str, knot_strerror(ret));
 					knot_pkt_free(reply);
 					net_close(net);
-					return 0;
+					return -1;
 				}
 			}
 
@@ -1021,7 +1023,7 @@ static int process_xfr_packet(const knot_pkt_t      *query,
 				    net->remote_str);
 				knot_pkt_free(reply);
 				net_close(net);
-				return 0;
+				return -1;
 			}
 
 			// Check for question sections equality.
@@ -1059,22 +1061,17 @@ static int process_xfr_packet(const knot_pkt_t      *query,
 	return 0;
 }
 
-static void process_xfr(const query_t *query)
+static int process_xfr(const query_t *query)
 {
 	knot_pkt_t *out_packet;
 	net_t      net;
 	int        ret;
 
-	if (query == NULL) {
-		DBG_NULL;
-		return;
-	}
-
 	// Create query packet.
 	out_packet = create_query_packet(query);
 	if (out_packet == NULL) {
 		ERR("can't create query packet\n");
-		return;
+		return -1;
 	}
 
 	// Sign the query.
@@ -1082,7 +1079,7 @@ static void process_xfr(const query_t *query)
 	ret = sign_query(out_packet, query, &sign_ctx);
 	if (ret != KNOT_EOK) {
 		ERR("can't sign the packet (%s)\n", knot_strerror(ret));
-		return;
+		return -1;
 	}
 
 	// Get connection parameters.
@@ -1104,7 +1101,7 @@ static void process_xfr(const query_t *query)
 	if (ret != KNOT_EOK) {
 		sign_context_deinit(&sign_ctx);
 		knot_pkt_free(out_packet);
-		return;
+		return -1;
 	}
 
 	// Loop over all resolved addresses for remote.
@@ -1130,6 +1127,8 @@ static void process_xfr(const query_t *query)
 	net_clean(&net);
 	sign_context_deinit(&sign_ctx);
 	knot_pkt_free(out_packet);
+
+	return ret;
 }
 
 int kdig_exec(const kdig_params_t *params)
@@ -1141,20 +1140,23 @@ int kdig_exec(const kdig_params_t *params)
 		return KNOT_EINVAL;
 	}
 
+	bool success = true;
+
 	// Loop over query list.
 	WALK_LIST(n, params->queries) {
 		query_t *query = (query_t *)n;
 
+		int ret = -1;
 		switch (query->operation) {
 		case OPERATION_QUERY:
-			process_query(query);
+			ret = process_query(query);
 			break;
 		case OPERATION_XFR:
-			process_xfr(query);
+			ret = process_xfr(query);
 			break;
 #if USE_DNSTAP
 		case OPERATION_LIST_DNSTAP:
-			process_dnstap(query);
+			ret = process_dnstap(query);
 			break;
 #endif // USE_DNSTAP
 		case OPERATION_LIST_SOA:
@@ -1164,11 +1166,16 @@ int kdig_exec(const kdig_params_t *params)
 			break;
 		}
 
+		// All operations must succeed.
+		if (ret != 0) {
+			success = false;
+		}
+
 		// If not last query, print separation.
 		if (n->next->next && params->config->style.format == FORMAT_FULL) {
 			printf("\n");
 		}
 	}
 
-	return KNOT_EOK;
+	return success ? KNOT_EOK : KNOT_ERROR;
 }
