@@ -1,4 +1,4 @@
-/*  Copyright (C) 2011 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,22 +13,9 @@
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
-/*!
- * \file kdig_exec.h
- *
- * \author Daniel Salzman <daniel.salzman@nic.cz>
- *
- * \brief kdig executives.
- *
- * \addtogroup knot_utils
- * @{
- */
 
 #pragma once
 
-#include "utils/common/params.h"
 #include "utils/kdig/kdig_params.h"
 
 int kdig_exec(const kdig_params_t *params);
-
-/*! @} */
