@@ -69,6 +69,7 @@ static const style_t DEFAULT_STYLE_DIG = {
 	},
 	.show_query = false,
 	.show_header = true,
+	.show_section = true,
 	.show_edns = true,
 	.show_question = true,
 	.show_answer = true,
@@ -353,6 +354,24 @@ static int opt_noheader(const char *arg, void *query)
 	return KNOT_EOK;
 }
 
+static int opt_comments(const char *arg, void *query)
+{
+	query_t *q = query;
+
+	q->style.show_section = true;
+
+	return KNOT_EOK;
+}
+
+static int opt_nocomments(const char *arg, void *query)
+{
+	query_t *q = query;
+
+	q->style.show_section = false;
+
+	return KNOT_EOK;
+}
+
 static int opt_opt(const char *arg, void *query)
 {
 	query_t *q = query;
@@ -993,19 +1012,19 @@ static int opt_noedns(const char *arg, void *query)
 	return KNOT_EOK;
 }
 
-static int opt_time(const char *arg, void *query)
+static int opt_timeout(const char *arg, void *query)
 {
 	query_t *q = query;
 
 	if (params_parse_wait(arg, &q->wait) != KNOT_EOK) {
-		ERR("invalid +time=%s\n", arg);
+		ERR("invalid +timeout=%s\n", arg);
 		return KNOT_EINVAL;
 	}
 
 	return KNOT_EOK;
 }
 
-static int opt_notime(const char *arg, void *query)
+static int opt_notimeout(const char *arg, void *query)
 {
 	query_t *q = query;
 
@@ -1152,6 +1171,10 @@ static const param_t kdig_opts2[] = {
 	{ "header",         ARG_NONE,     opt_header },
 	{ "noheader",       ARG_NONE,     opt_noheader },
 
+	{ "comments",       ARG_NONE,     opt_comments },
+	{ "nocomments",     ARG_NONE,     opt_nocomments },
+
+	{ "opt",            ARG_NONE,     opt_opt },
 	{ "opt",            ARG_NONE,     opt_opt },
 	{ "noopt",          ARG_NONE,     opt_noopt },
 
@@ -1228,8 +1251,8 @@ static const param_t kdig_opts2[] = {
 	{ "edns",           ARG_OPTIONAL, opt_edns },
 	{ "noedns",         ARG_NONE,     opt_noedns },
 
-	{ "time",           ARG_REQUIRED, opt_time },
-	{ "notime",         ARG_NONE,     opt_notime },
+	{ "timeout",        ARG_REQUIRED, opt_timeout },
+	{ "notimeout",      ARG_NONE,     opt_notimeout },
 
 	{ "retry",          ARG_REQUIRED, opt_retry },
 	{ "noretry",        ARG_NONE,     opt_noretry },
@@ -1889,6 +1912,7 @@ static void print_help(void)
 	       "       +[no]all                  Show all packet sections.\n"
 	       "       +[no]qr                   Show query packet.\n"
 	       "       +[no]header               Show packet header.\n"
+	       "       +[no]comments             Show commented section names.\n"
 	       "       +[no]opt                  Show EDNS pseudosection.\n"
 	       "       +[no]question             Show question section.\n"
 	       "       +[no]answer               Show answer section.\n"
@@ -1913,7 +1937,7 @@ static void print_help(void)
 	       "       +[no]alignment[=N]        Pad with EDNS(0) to blocksize (%u or specify size).\n"
 	       "       +[no]subnet=SUBN          Set EDNS(0) client subnet addr/prefix.\n"
 	       "       +[no]edns[=N]             Use EDNS(=version).\n"
-	       "       +[no]time=T               Set wait for reply interval in seconds.\n"
+	       "       +[no]timeout=T            Set wait for reply interval in seconds.\n"
 	       "       +[no]retry=N              Set number of retries.\n"
 	       "       +[no]cookie=HEX           Attach EDNS(0) cookie to the query.\n"
 	       "       +[no]badcookie            Repeat a query with the correct cookie.\n"
