@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 #include <stdint.h>
 #include <time.h>
 
-#include "dnssec/binary.h"
+#include "libdnssec/binary.h"
 #include "knot/dnssec/kasp/policy.h"
 
 /*!
