@@ -27,6 +27,7 @@
 #include "contrib/tolower.h"
 #include "contrib/wire_ctx.h"
 #include "libdnssec/error.h"
+#include "libdnssec/shared/hex.h"
 #include "libdnssec/shared/shared.h"
 #include "knot/dnssec/kasp/policy.h"
 #include "knot/dnssec/zone-keys.h"
@@ -190,6 +191,10 @@ int keymgr_generate_key(kdnssec_ctx_t *ctx, int argc, char *argv[])
 		return ret;
 	}
 
+	if ((flags & DNSKEY_GENERATE_KSK) && gen_timing.ready == infty) {
+		gen_timing.ready = gen_timing.active;
+	}
+
 	if (keysize > 0) {
 		if ((flags & DNSKEY_GENERATE_KSK)) {
 			ctx->policy->ksk_size = keysize;
@@ -539,6 +544,54 @@ int keymgr_import_pkcs11(kdnssec_ctx_t *ctx, const char *key_id, int argc, char
 	return import_key(ctx, KEYSTORE_BACKEND_PKCS11, key_id, argc, argv);
 }
 
+int keymgr_nsec3_salt_print(kdnssec_ctx_t *ctx)
+{
+	dnssec_binary_t salt_bin;
+	knot_time_t created;
+	int ret = kasp_db_load_nsec3salt(*ctx->kasp_db, ctx->zone->dname,
+	                                 &salt_bin, &created);
+	switch (ret) {
+	case KNOT_EOK:
+		printf("Current salt: ");
+		if (salt_bin.size == 0) {
+			printf("-");
+		}
+		for (size_t i = 0; i < salt_bin.size; i++) {
+			printf("%02X", (unsigned)salt_bin.data[i]);
+		}
+		printf("\n");
+		free(salt_bin.data);
+		break;
+	case KNOT_ENOENT:
+		printf("-- no salt --\n");
+		ret = KNOT_EOK;
+		break;
+	}
+	return ret;
+}
+
+int keymgr_nsec3_salt_set(kdnssec_ctx_t *ctx, const char *new_salt)
+{
+	assert(new_salt);
+
+	dnssec_binary_t salt_bin = { 0 };
+	if (strcmp(new_salt, "-") != 0 &&
+	    hex_to_bin(new_salt, &salt_bin) != DNSSEC_EOK) {
+		return KNOT_EMALF;
+	}
+	if (salt_bin.size != ctx->policy->nsec3_salt_length) {
+		printf("Warning: specified salt doesn't match configured "
+		       "salt length (%d).\n",
+		       (int)ctx->policy->nsec3_salt_length);
+	}
+	int ret = kasp_db_store_nsec3salt(*ctx->kasp_db, ctx->zone->dname,
+	                                  &salt_bin, knot_time());
+	if (salt_bin.size > 0) {
+		free(salt_bin.data);
+	}
+	return ret;
+}
+
 static void print_tsig(dnssec_tsig_algorithm_t mac, const char *name,
 		       const dnssec_binary_t *secret)
 {
