@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -73,11 +73,11 @@ static void print_changeset(const changeset_t *chs, bool color)
 	printf(color ? YLW : "");
 	if (chs->soa_from == NULL) {
 		printf(";; Zone-in-journal, serial: %u\n",
-		       knot_soa_serial(&chs->soa_to->rrs));
+		       knot_soa_serial(chs->soa_to->rrs.rdata));
 	} else {
 		printf(";; Changes between zone versions: %u -> %u\n",
-		       knot_soa_serial(&chs->soa_from->rrs),
-		       knot_soa_serial(&chs->soa_to->rrs));
+		       knot_soa_serial(chs->soa_from->rrs.rdata),
+		       knot_soa_serial(chs->soa_to->rrs.rdata));
 	}
 	changeset_print(chs, stdout, color);
 }
@@ -110,7 +110,7 @@ static int rrtypelist_callback(zone_node_t *node, void *data)
 	for (int i = 0; i < node->rrset_count; i++) {
 		knot_rrset_t rrset = node_rrset_at(node, i);
 		rrtypelist_add(ctx->arr, rrset.type);
-		*ctx->counter += rrset.rrs.rr_count;
+		*ctx->counter += rrset.rrs.count;
 	}
 	return KNOT_EOK;
 }
@@ -127,11 +127,11 @@ static void print_changeset_debugmode(const changeset_t *chs)
 	(void)zone_contents_nsec3_apply(chs->add, rrtypelist_callback, &ctx_plus);
 
 	if (chs->soa_from == NULL) {
-		printf("Zone-in-journal %u  +++: %zu\t size: %zu\t", knot_soa_serial(&chs->soa_to->rrs),
+		printf("Zone-in-journal %u  +++: %zu\t size: %zu\t", knot_soa_serial(chs->soa_to->rrs.rdata),
 		       count_plus, changeset_serialized_size(chs));
 	} else {
-		printf("%u -> %u  ---: %zu\t  +++: %zu\t size: %zu\t", knot_soa_serial(&chs->soa_from->rrs),
-		       knot_soa_serial(&chs->soa_to->rrs), count_minus, count_plus, changeset_serialized_size(chs));
+		printf("%u -> %u  ---: %zu\t  +++: %zu\t size: %zu\t", knot_soa_serial(chs->soa_from->rrs.rdata),
+		       knot_soa_serial(chs->soa_to->rrs.rdata), count_minus, count_plus, changeset_serialized_size(chs));
 	}
 
 	char temp[100];
@@ -184,8 +184,9 @@ int print_journal(char *path, knot_dname_t *name, uint32_t limit, bool color, bo
 
 	bool has_bootstrap;
 	kserial_t merged_serial, serial_from, last_flushed, serial_to;
-	uint64_t occupied;
-	journal_metadata_info(j, &has_bootstrap, &merged_serial, &serial_from, &last_flushed, &serial_to, &occupied);
+	uint64_t occupied, occupied_all;
+	journal_metadata_info(j, &has_bootstrap, &merged_serial, &serial_from,
+			      &last_flushed, &serial_to, &occupied, &occupied_all);
 
 	bool alternative_from = (has_bootstrap || merged_serial.valid);
 	bool is_empty = (!alternative_from && !serial_from.valid);
@@ -225,16 +226,24 @@ int print_journal(char *path, knot_dname_t *name, uint32_t limit, bool color, bo
 	if (debugmode) {
 		if ((alternative_from && serial_from.valid) ||
 		    kserial_equal(serial_from, last_flushed)) {
-			printf("---- Additional history ----\n");
 			init_list(&db);
 
 			ret = journal_load_changesets(j, &db, serial_from.serial);
-			if (ret != KNOT_EOK) {
+			switch (ret) {
+			case KNOT_EOK:
+				printf("---- Additional history ----\n");
+				break;
+			case KNOT_ENOENT:
+				printf("---- No additional history ----\n");
+				ret = KNOT_EOK;
+				break;
+			default:
 				goto pj_finally;
 			}
 			WALK_LIST(chs, db) {
 				print_changeset_debugmode(chs);
-				if (last_flushed.valid && serial_equal(knot_soa_serial(&chs->soa_from->rrs), last_flushed.serial)) {
+				if (last_flushed.valid &&
+				    serial_equal(knot_soa_serial(chs->soa_from->rrs.rdata), last_flushed.serial)) {
 					break;
 				}
 			}
@@ -245,7 +254,8 @@ int print_journal(char *path, knot_dname_t *name, uint32_t limit, bool color, bo
 	}
 
 	if (debugmode) {
-		printf("Occupied: %"PRIu64" KiB\n", occupied / 1024);
+		printf("Occupied this zone (approx): %"PRIu64" KiB\n", occupied / 1024);
+		printf("Occupied all zones together: %"PRIu64" KiB\n", occupied_all / 1024);
 	}
 
 pj_finally:
@@ -281,8 +291,9 @@ int list_zones(char *path)
 	ret = journal_db_list_zones(&jdb, &zones);
 	if (ret == KNOT_EOK) {
 		ptrnode_t *zn;
+		char buff[KNOT_DNAME_TXT_MAXLEN + 1];
 		WALK_LIST(zn, zones) {
-			printf("%s\n", (char *)zn->d);
+			printf("%s\n", knot_dname_to_str(buff, (knot_dname_t *)zn->d, sizeof(buff)));
 			free(zn->d);
 		}
 		ptrlist_free(&zones, NULL);
@@ -343,6 +354,7 @@ int main(int argc, char *argv[])
 	switch (argc - optind) {
 	case 2:
 		name = knot_dname_from_str_alloc(argv[optind + 1]);
+		knot_dname_to_lower(name);
 		// FALLTHROUGH
 	case 1:
 		db = argv[optind];
