@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,19 +13,11 @@
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
-/*!
- * \file
- *
- * \brief Memory estimation for zone files.
- *
- * \addtogroup knot_utils
- * @{
- */
 
 #pragma once
 
 #include "contrib/qp-trie/trie.h"
-#include "zscanner/scanner.h"
+#include "libzscanner/scanner.h"
 
 /*!
  * \brief Memory estimation context.
@@ -70,5 +62,3 @@ void estimator_rrset_memsize_wrap(zs_scanner_t *scanner);
  * \param data Unused variable.
  */
 int estimator_free_trie_node(trie_val_t *val, void *data);
-
-/*! @} */
