@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,16 +25,17 @@
 #include <sys/stat.h>
 #include <urcu.h>
 
-#ifdef HAVE_CAP_NG_H
+#ifdef ENABLE_CAP_NG
 #include <cap-ng.h>
-#endif /* HAVE_CAP_NG_H */
+#endif
 
 #ifdef ENABLE_SYSTEMD
 #include <systemd/sd-daemon.h>
 #endif
 
-#include "dnssec/crypto.h"
+#include "libdnssec/crypto.h"
 #include "libknot/libknot.h"
+#include "contrib/strtonum.h"
 #include "knot/ctl/process.h"
 #include "knot/conf/conf.h"
 #include "knot/conf/migration.h"
@@ -100,12 +101,15 @@ static int make_daemon(int nochdir, int noclose)
 		}
 
 		if (dup2(fd, STDIN_FILENO) < 0) {
+			close(fd);
 			return errno;
 		}
 		if (dup2(fd, STDOUT_FILENO) < 0) {
+			close(fd);
 			return errno;
 		}
 		if (dup2(fd, STDERR_FILENO) < 0) {
+			close(fd);
 			return errno;
 		}
 	}
@@ -138,7 +142,7 @@ static void handle_signal(int signum)
 	case SIGINT:
 	case SIGTERM:
 		if (sig_req_stop) {
-			abort();
+			exit(EXIT_FAILURE);
 		}
 		sig_req_stop = true;
 		break;
@@ -179,42 +183,23 @@ static void enable_signals(void)
 	pthread_sigmask(SIG_UNBLOCK, &mask, NULL);
 }
 
-/*! \brief POSIX 1003.1e capabilities. */
-static void setup_capabilities(void)
+/*! \brief Drop POSIX 1003.1e capabilities. */
+static void drop_capabilities(void)
 {
-#ifdef HAVE_CAP_NG_H
+#ifdef ENABLE_CAP_NG
 	/* Drop all capabilities. */
 	if (capng_have_capability(CAPNG_EFFECTIVE, CAP_SETPCAP)) {
 		capng_clear(CAPNG_SELECT_BOTH);
 
-		/* Retain ability to set capabilities and FS access. */
-		capng_type_t tp = CAPNG_EFFECTIVE|CAPNG_PERMITTED;
-		capng_update(CAPNG_ADD, tp, CAP_SETPCAP);
-		capng_update(CAPNG_ADD, tp, CAP_DAC_OVERRIDE);
-		capng_update(CAPNG_ADD, tp, CAP_CHOWN); /* Storage ownership. */
-
-		/* Allow binding to privileged ports.
-		 * (Not inheritable)
-		 */
-		capng_update(CAPNG_ADD, tp, CAP_NET_BIND_SERVICE);
-
-		/* Allow setuid/setgid. */
-		capng_update(CAPNG_ADD, tp, CAP_SETUID);
-		capng_update(CAPNG_ADD, tp, CAP_SETGID);
-
-		/* Allow priorities changing. */
-		capng_update(CAPNG_ADD, tp, CAP_SYS_NICE);
-
 		/* Apply. */
 		if (capng_apply(CAPNG_SELECT_BOTH) < 0) {
 			log_error("failed to set process capabilities (%s)",
 			          strerror(errno));
 		}
 	} else {
-		log_info("user UID %d is not allowed to set capabilities, "
-		         "skipping", getuid());
+		log_info("process not allowed to set capabilities, skipping");
 	}
-#endif /* HAVE_CAP_NG_H */
+#endif /* ENABLE_CAP_NG */
 }
 
 /*! \brief Event loop listening for signals and remote commands. */
@@ -242,6 +227,7 @@ static void event_loop(server_t *server, const char *socket)
 		listen = strdup(socket);
 	}
 	if (listen == NULL) {
+		knot_ctl_free(ctl);
 		log_fatal("control, empty socket path");
 		return;
 	}
@@ -297,17 +283,20 @@ static void print_help(void)
 	printf("Usage: %s [parameters]\n"
 	       "\n"
 	       "Parameters:\n"
-	       " -c, --config <file>     Use a textual configuration file.\n"
-	       "                           (default %s)\n"
-	       " -C, --confdb <dir>      Use a binary configuration database directory.\n"
-	       "                           (default %s)\n"
-	       " -s, --socket <path>     Use a remote control UNIX socket path.\n"
-	       "                           (default %s)\n"
-	       " -d, --daemonize=[dir]   Run the server as a daemon (with new root directory).\n"
-	       " -v, --verbose           Enable debug output.\n"
-	       " -h, --help              Print the program help.\n"
-	       " -V, --version           Print the program version.\n",
-	       PROGRAM_NAME, CONF_DEFAULT_FILE, CONF_DEFAULT_DBDIR, RUN_DIR "/knot.sock");
+	       " -c, --config <file>        Use a textual configuration file.\n"
+	       "                             (default %s)\n"
+	       " -C, --confdb <dir>         Use a binary configuration database directory.\n"
+	       "                             (default %s)\n"
+	       " -m, --max-conf-size <MiB>  Set maximum configuration size (max 10000 MiB).\n"
+	       "                             (default %d MiB)\n"
+	       " -s, --socket <path>        Use a remote control UNIX socket path.\n"
+	       "                             (default %s)\n"
+	       " -d, --daemonize=[dir]      Run the server as a daemon (with new root directory).\n"
+	       " -v, --verbose              Enable debug output.\n"
+	       " -h, --help                 Print the program help.\n"
+	       " -V, --version              Print the program version.\n",
+	       PROGRAM_NAME, CONF_DEFAULT_FILE, CONF_DEFAULT_DBDIR,
+	       CONF_MAPSIZE, RUN_DIR "/knot.sock");
 }
 
 static void print_version(void)
@@ -315,7 +304,7 @@ static void print_version(void)
 	printf("%s (Knot DNS), version %s\n", PROGRAM_NAME, PACKAGE_VERSION);
 }
 
-static int set_config(const char *confdb, const char *config)
+static int set_config(const char *confdb, const char *config, size_t max_conf_size)
 {
 	if (config != NULL && confdb != NULL) {
 		log_fatal("ambiguous configuration source");
@@ -343,7 +332,7 @@ static int set_config(const char *confdb, const char *config)
 
 	/* Open confdb. */
 	conf_t *new_conf = NULL;
-	int ret = conf_new(&new_conf, conf_schema, confdb, CONF_FREQMODULES);
+	int ret = conf_new(&new_conf, conf_schema, confdb, max_conf_size, CONF_FREQMODULES);
 	if (ret != KNOT_EOK) {
 		log_fatal("failed to open configuration database '%s' (%s)",
 		          (confdb != NULL) ? confdb : "", knot_strerror(ret));
@@ -367,10 +356,6 @@ static int set_config(const char *confdb, const char *config)
 		log_error("failed to migrate configuration (%s)", knot_strerror(ret));
 	}
 
-	/* Activate global query modules. */
-	conf_activate_modules(new_conf, NULL, new_conf->query_modules,
-	                      &new_conf->query_plan);
-
 	/* Update to the new config. */
 	conf_update(new_conf, CONF_UPD_FNONE);
 
@@ -411,25 +396,27 @@ int main(int argc, char **argv)
 	bool daemonize = false;
 	const char *config = NULL;
 	const char *confdb = NULL;
+	size_t max_conf_size = (size_t)CONF_MAPSIZE * 1024 * 1024;
 	const char *daemon_root = "/";
 	char *socket = NULL;
 	bool verbose = false;
 
 	/* Long options. */
 	struct option opts[] = {
-		{ "config",    required_argument, NULL, 'c' },
-		{ "confdb",    required_argument, NULL, 'C' },
-		{ "socket",    required_argument, NULL, 's' },
-		{ "daemonize", optional_argument, NULL, 'd' },
-		{ "verbose",   no_argument,       NULL, 'v' },
-		{ "help",      no_argument,       NULL, 'h' },
-		{ "version",   no_argument,       NULL, 'V' },
+		{ "config",        required_argument, NULL, 'c' },
+		{ "confdb",        required_argument, NULL, 'C' },
+		{ "max-conf-size", required_argument, NULL, 'm' },
+		{ "socket",        required_argument, NULL, 's' },
+		{ "daemonize",     optional_argument, NULL, 'd' },
+		{ "verbose",       no_argument,       NULL, 'v' },
+		{ "help",          no_argument,       NULL, 'h' },
+		{ "version",       no_argument,       NULL, 'V' },
 		{ NULL }
 	};
 
 	/* Parse command line arguments. */
-	int opt = 0, li = 0;
-	while ((opt = getopt_long(argc, argv, "c:C:s:dvhV", opts, &li)) != -1) {
+	int opt = 0;
+	while ((opt = getopt_long(argc, argv, "c:C:m:s:dvhV", opts, NULL)) != -1) {
 		switch (opt) {
 		case 'c':
 			config = optarg;
@@ -437,6 +424,14 @@ int main(int argc, char **argv)
 		case 'C':
 			confdb = optarg;
 			break;
+		case 'm':
+			if (str_to_size(optarg, &max_conf_size, 1, 10000) != KNOT_EOK) {
+				print_help();
+				return EXIT_FAILURE;
+			}
+			/* Convert to bytes. */
+			max_conf_size *= 1024 * 1024;
+			break;
 		case 's':
 			socket = optarg;
 			break;
@@ -488,9 +483,6 @@ int main(int argc, char **argv)
 	/* Initialize pseudorandom number generator. */
 	srand(time(NULL));
 
-	/* POSIX 1003.1e capabilities. */
-	setup_capabilities();
-
 	/* Initialize logging subsystem. */
 	log_init();
 	if (verbose) {
@@ -498,7 +490,7 @@ int main(int argc, char **argv)
 	}
 
 	/* Set up the configuration */
-	int ret = set_config(confdb, config);
+	int ret = set_config(confdb, config, max_conf_size);
 	if (ret != KNOT_EOK) {
 		log_close();
 		return EXIT_FAILURE;
@@ -534,6 +526,13 @@ int main(int argc, char **argv)
 		return EXIT_FAILURE;
 	}
 
+	/* Drop POSIX capabilities. */
+	drop_capabilities();
+
+	/* Activate global query modules. */
+	conf_activate_modules(conf(), NULL, conf()->query_modules,
+	                      &conf()->query_plan);
+
 	/* Check and create PID file. */
 	long pid = (long)getpid();
 	if (daemonize) {
