@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 #include <sys/socket.h>
 #include <unistd.h>
 
-#include "dnssec/random.h"
+#include "libdnssec/random.h"
 #include "utils/knsupdate/knsupdate_exec.h"
 #include "utils/common/exec.h"
 #include "utils/common/msg.h"
@@ -151,7 +151,7 @@ static bool dname_isvalid(const char *lp)
 	if (dn == NULL) {
 		return false;
 	}
-	knot_dname_free(&dn, NULL);
+	knot_dname_free(dn, NULL);
 	return true;
 }
 
@@ -197,7 +197,7 @@ static int parse_partial_rr(zs_scanner_t *s, const char *lp, unsigned flags)
 
 	s->r_owner_length = knot_dname_size(owner);
 	memcpy(s->r_owner, owner, s->r_owner_length);
-	knot_dname_free(&owner, NULL);
+	knot_dname_free(owner, NULL);
 
 	/* Append origin if not FQDN. */
 	if (!fqdn) {
@@ -342,24 +342,23 @@ static srv_info_t *parse_host(const char *lp, const char* default_port)
 static int rr_list_append(zs_scanner_t *s, list_t *target_list, knot_mm_t *mm)
 {
 	knot_rrset_t *rr = knot_rrset_new(s->r_owner, s->r_type, s->r_class,
-	                                  NULL);
+	                                  s->r_ttl, NULL);
 	if (!rr) {
 		DBG("%s: failed to create rrset\n", __func__);
 		return KNOT_ENOMEM;
 	}
 
 	/* Create RDATA. */
-	int ret = knot_rrset_add_rdata(rr, s->r_data, s->r_data_length,
-	                               s->r_ttl, NULL);
+	int ret = knot_rrset_add_rdata(rr, s->r_data, s->r_data_length, NULL);
 	if (ret != KNOT_EOK) {
 		DBG("%s: failed to set rrset from wire (%s)\n",
 		    __func__, knot_strerror(ret));
-		knot_rrset_free(&rr, NULL);
+		knot_rrset_free(rr, NULL);
 		return ret;
 	}
 
 	if (ptrlist_add(target_list, rr, mm) == NULL) {
-		knot_rrset_free(&rr, NULL);
+		knot_rrset_free(rr, NULL);
 		return KNOT_ENOMEM;
 	}
 
@@ -397,7 +396,7 @@ static int build_query(knsupdate_params_t *params)
 	knot_dname_t *qname = knot_dname_from_str_alloc(params->zone);
 	int ret = knot_pkt_put_question(query, qname, params->class_num,
 	                                params->type_num);
-	knot_dname_free(&qname, NULL);
+	knot_dname_free(qname, NULL);
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
