@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -80,7 +80,7 @@ int zone_check(const char *zone_file, const knot_dname_t *zone_name,
 	if (contents == NULL && !stats.handler.fatal_error) {
 		return KNOT_ERROR;
 	}
-	zone_contents_deep_free(&contents);
+	zone_contents_deep_free(contents);
 
 	if (stats.error_count > 0) {
 		print_statistics(&stats);
